/* $Id: AlignPrim.java,v 1.1.1.1 2001/03/21 08:24:12 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.align;

import	nts.base.Dimen;
import	nts.builder.Builder;
import	nts.builder.ListBuilder;
import	nts.command.Token;
import	nts.command.TokenList;
import	nts.command.FrozenToken;
import	nts.command.Group;
import	nts.command.Closing;
import	nts.command.Command;
import	nts.command.Primitive;
import	nts.typo.Action;
import	nts.typo.BuilderPrim;
import	nts.math.DisplayBuilder;

public abstract class	AlignPrim	extends BuilderPrim {

    private final TokenList.Inserter	everyCr;
    private final Token			frozenCr;
    private final Token			frozenEndTemplate;

    public AlignPrim(String name, TokenList.Inserter everyCr,
		     Primitive carRet, Command endv) {
	super(name); this.everyCr = everyCr;
	frozenCr = new FrozenToken(carRet);
	frozenEndTemplate = new FrozenToken("endtemplate",
		new EndTemplate(new FrozenToken("endtemplate", endv)));
    }

    /* STRANGE
     * Why is the outer builder pushed before scanning the alignment spec?
     * It is inconsistent with \hbox, etc. and makes the code more complicated.
     */
    public final Action		NORMAL = new Action() {
	public void	exec(Builder bld, Token src) {
	    //XXX[774] first align_group (scan_spec(align_group, false);)
	    //XXX[774] pushLevel(new SimpleGroup()); ???
	    ListBuilder		builder = makeOuterBuilder(currLineNumber());
	    Builder.push(builder);
	    Alignment	align = scanAlignSpec(builder);
	    pushLevel(new AlignGroup(align));
	    align.start(src);
	}
    };

    protected class	DisplayedClosing	extends Closing {
	
	/* TeXtp[774,776] */
	public void		exec(Group grp, Token src) {
	    if (!(getBld() instanceof DisplayBuilder))
		throw new RuntimeException(
			    "non-display builder for displayed halign");
	    Builder	bld = getBld();
	    if (!bld.isEmpty()) {
		error("ImproperAlignInFormula", AlignPrim.this);
		Builder.pop();
		Builder.push(new DisplayBuilder(bld.getStartLine()));
	    }
	    ListBuilder		builder = makeOuterBuilder(currLineNumber());
	    Builder.push(builder);
	    Alignment	align = scanAlignSpec(builder);
	    pushLevel(new DispAlignGroup(align));
	    align.start(src);
	}

    }

    public Closing	makeDisplayedClosing()
	{ return new DisplayedClosing(); }

    /* STRANGE
     * Why is the starting line number (for outer builder) before
     * size specification and left brace?
     */
    /* TeXtp[774] */
    protected Alignment		scanAlignSpec(ListBuilder builder) {
	Dimen		size = Dimen.ZERO;
	boolean		exactly = false;
	if (scanKeyword("to")) { size = scanDimen(); exactly = true; }
	else if (scanKeyword("spread")) size = scanDimen();
	scanLeftBrace();
	return makeAlignment(size, exactly, everyCr, frozenCr,
			     frozenEndTemplate, builder);
    }

    protected abstract ListBuilder	makeOuterBuilder(int lineNo);
    protected abstract Alignment	makeAlignment(
						Dimen size, boolean exactly,
						TokenList.Inserter everyCr,
						Token frzCr, Token frzEndt,
						ListBuilder builder);

}
