/* $Id: AlignSizesMatrix.java,v 1.1.1.1 2000/08/10 03:28:29 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.align;

import	java.util.Vector;
import	java.util.HashMap;
import	nts.base.IntPairKey;
import	nts.base.Dimen;

public class	AlignSizesMatrix {

    private final Vector	diagonal;
    private final HashMap	matrix;

    public AlignSizesMatrix() {
	diagonal = new Vector();
	matrix = new HashMap();
    }

    public AlignSizesMatrix(int initialSize) {
	diagonal = new Vector(initialSize);
	matrix = new HashMap();
    }

    public int		size() { return diagonal.size(); }

    public Dimen	get(int i) {
	return (i < diagonal.size()) ? (Dimen) diagonal.get(i)
				     : Dimen.NULL;
    }

    public Dimen	get(int i, int j) {
	return (i != j) ? (Dimen) matrix.get(new IntPairKey(i, j))
			: get(i);
    }

    public Dimen	set(int i, int j, Dimen value) {
	if (i != j) return (Dimen) matrix.put(new IntPairKey(i, j), value);
	if (i >= diagonal.size()) diagonal.setSize(i + 1);
	return (Dimen) diagonal.set(i, value);
    }

    public void		setMax(int i, int j, Dimen value) {
	if (i != j) {
	    IntPairKey		key = new IntPairKey(i, j);
	    Dimen		old = (Dimen) matrix.get(key);
	    if (old == Dimen.NULL || value.moreThan(old))
		matrix.put(key, value);
	} else if (i >= diagonal.size())
	    { diagonal.setSize(i + 1); diagonal.set(i, value); }
	else {
	    Dimen		old = (Dimen) diagonal.get(i);
	    if (old == Dimen.NULL || value.moreThan(old))
		diagonal.set(i, value);
	}
    }

}
