/* $Id: AnyUnsetNode.java,v 1.1.1.1 2001/03/22 05:38:30 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.align;

import	nts.base.Dimen;
import	nts.base.Glue;
import	nts.io.Log;
import	nts.io.CntxLog;
import	nts.node.GlueSetting;
import	nts.node.BoxSizes;
import	nts.node.NodeList;
import	nts.node.AnyBoxedNode;
import	nts.node.SizesEvaluator;

public class	AnyUnsetNode	extends AnyBoxedNode {
    /* corresponding to unset_node */

    protected final NodeList		list;
    protected final int			spanCount;
    protected final Dimen		stretch;
    protected final byte		strOrder;
    protected final Dimen		shrink;
    protected final byte		shrOrder;

    public AnyUnsetNode(BoxSizes sizes, NodeList list, int spanCount,
			Dimen stretch, byte strOrder,
			Dimen shrink, byte shrOrder) {
	super(sizes); this.list = list; this.spanCount = spanCount;
	this.stretch = stretch; this.strOrder = strOrder;
	this.shrink = shrink; this.shrOrder = shrOrder;
    }

    public AnyUnsetNode(BoxSizes sizes, NodeList list) {
	this(sizes, list, 0, Dimen.ZERO, Glue.NORMAL,
			     Dimen.ZERO, Glue.NORMAL);
    }

    public AnyUnsetNode() { this(BoxSizes.ZERO, NodeList.EMPTY); }

    public NodeList	getList() { return list; }
    public int		getSpanCount() { return spanCount; }

    /* TeXtp[184,185] */
    public void		addOn(Log log, CntxLog cntx) {
	log.addEsc(getDesc()).add(sizes);
    	StringBuffer	buf = new StringBuffer(80);
	if (spanCount != 0)
	    buf.append(" (").append(spanCount + 1).append(" columns)");
	if (!stretch.isZero())
	    Glue.append(buf.append(", stretch "), stretch, strOrder, null);
	if (!shrink.isZero())
	    Glue.append(buf.append(", shrink "), shrink, shrOrder, null);
	log.add(buf.toString());
	cntx.addOn(log, list.nodes());
    }

    public String	getDesc() { return "unsetbox"; }

    /* TeXtp[810,811] */
    public GlueSetting	getSetting(Dimen excess) {
	SizesEvaluator		pack = new SizesEvaluator();
	pack.addStretch(stretch, strOrder);
	pack.addShrink(shrink, shrOrder);
	pack.evaluate(excess, false);
	return pack.getSetting();
    }

}
