/* $Id: Preamble.java,v 1.1.1.1 2000/08/07 03:04:04 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.align;

import	nts.base.Glue;
import	nts.command.Token;
import	nts.command.TokenList;

public class	Preamble {

    public static final Entry	NULL_ENTRY = null;

    public static class	Entry {
	private TokenList	uPart;
	private TokenList	vPart;
	private Glue		skip;

	public Entry(TokenList uPart, TokenList vPart, Glue skip)
	    { this.uPart = uPart; this.vPart = vPart; this.skip = skip; }

    }

    public final Glue		firstSkip;
    public final String		skipName;
    public final Token		endTemplate;
    private final Entry[]	records;
    private final int		loopIndex;

    public Preamble(Glue firstSkip, String skipName, Token endTemplate,
		    Entry[] records, int loopIndex) {
	this.firstSkip = firstSkip; this.skipName = skipName;
	this.endTemplate = endTemplate;
	this.records = records; this.loopIndex = loopIndex;
	if (loopIndex >= records.length)
	    throw new RuntimeException("invalid Preamble loopIndex");
    }

    public int		length() { return records.length; }
    public boolean	periodic() { return (loopIndex >= 0); }

    public boolean	hasRecord(int i)
	{ return (i >= 0 && (i < records.length || loopIndex >= 0)); }

    public TokenList	getUPart(int i) {
	Entry		ent = getEntry(i);
	return (ent != NULL_ENTRY) ? ent.uPart : TokenList.NULL;
    }

    public TokenList	getVPart(int i) {
	Entry		ent = getEntry(i);
	return (ent != NULL_ENTRY) ? ent.vPart : TokenList.NULL;
    }

    public Glue		getSkip(int i) {
	Entry		ent = getEntry(i);
	return (ent != NULL_ENTRY) ? ent.skip : Glue.NULL;
    }

    private Entry	getEntry(int i) {
	if (i >= 0) {
	    if (i < records.length) return records[i];
	    else if (loopIndex >= 0)
		return records[loopIndex + (i - loopIndex)
					 % (records.length - loopIndex)];
	}
	return NULL_ENTRY;
    }

/*
    public void		trace() {
	System.err.println("=== Preamble ===");
	System.err.println("firstSkip = " + firstSkip);
	System.err.println("skipName = " + skipName);
	System.err.println("endTemplate = " + endTemplate);
	for (int i = 0; i < records.length; i++) {
	    System.err.println("index = " + i);
	    System.err.println("uPart = " + records[i].uPart);
	    System.err.println("vPart = " + records[i].vPart);
	    System.err.println("skip = " + records[i].skip);
	}
	System.err.println("loopIndex = " + loopIndex);
	System.err.println();
    }
*/

}
