/* $Id: PreambleScanning.java,v 1.1.1.1 2001/03/20 03:02:17 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.align;

import	java.util.Vector;
import	nts.base.Glue;
import	nts.command.Token;
import	nts.command.TokenList;
import	nts.command.RightBraceToken;
import	nts.command.BraceNesting;
import	nts.command.BaseToksChecker;
import	nts.command.Command;

public class	PreambleScanning	extends BaseToksChecker
					implements BraceNesting {

    private Vector		entries = new Vector();
    private TokenList.Buffer	buf = new TokenList.Buffer();
    private TokenList		uPart = TokenList.NULL;
    private int			loopIndex = -1;
    private int			braceNesting = 0;
    private final TokenList	crAndBrace;

    public PreambleScanning(Token source, Token frozenCr) {
	super("OuterInPreamble", "EOFinPreamble", source);
	Token[]		toks = { frozenCr, RightBraceToken.TOKEN };
	crAndBrace = new TokenList(toks);
    }

    public boolean	balanced() { return (braceNesting == 0); }
    public void		append(Token tok) { buf.append(tok); }

    public void		finishHalf()
	{ uPart = buf.toTokenList(); buf = new TokenList.Buffer(); }

    public void		finishRecord(Glue skip) {
	entries.add(new Preamble.Entry(uPart, buf.toTokenList(), skip));
	uPart = TokenList.NULL; buf = new TokenList.Buffer();
    }

    public boolean	setLoopIndex() {
	if (loopIndex >= 0) return false;
	loopIndex = entries.size();
	return true;
    }

    public Preamble	toPreamble(Glue firstSkip, String skipName,
				   Token endTemplate) {
	Preamble.Entry[]	records = new Preamble.Entry[entries.size()];
	entries.copyInto(records);
	return new Preamble(firstSkip, skipName, endTemplate,
			    records, loopIndex);
    }

    public void		adjust(int count) { braceNesting += count; }

    /* TeXtp[339] */	//XXX align_state [339]
    protected void	tryToFix() { Command.insertList(crAndBrace); }

    protected void	reportRunAway() { Command.runAway("preamble", buf); }

}
