/* $Id: VertAlignment.java,v 1.1.1.1 2001/03/21 08:16:51 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.align;

import	nts.base.Dimen;
import	nts.base.Glue;
import	nts.io.Log;
import	nts.node.GlueSetting;
import	nts.node.BoxSizes;
import	nts.node.NodeList;
import	nts.node.NodeEnum;
import	nts.node.SizesSummarizer;
import	nts.node.VertIterator;
import	nts.node.AnyBoxNode;
import	nts.node.VBoxNode;
import	nts.node.AnySkipNode;
import	nts.node.VSkipNode;
import	nts.node.NamedVSkipNode;
import	nts.builder.Builder;
import	nts.builder.ListBuilder;
import	nts.builder.VBoxBuilder;
import	nts.command.Token;
import	nts.command.TokenList;
import	nts.typo.TypoCommand;

public class	VertAlignment	extends Alignment {

    protected final ListBuilder		builder;

    public VertAlignment(Dimen size, boolean exactly,
			 TokenList.Inserter everyCr, Token frzCr,
			 Token frzEndt, ListBuilder builder) {
	super(size, exactly, everyCr, frzCr, frzEndt);
	this.builder = builder;
    }

    /* TeXtp[785] */
    protected void	startNonAligned() {
	super.startNonAligned();
	TypoCommand.getTypoConfig().resetParagraph();
    }

    /* TeXtp[787] */
    protected void	startSpan(int index) {
	super.startSpan(index);
	TypoCommand.getTypoConfig().resetParagraph();
    }

    protected VBoxBuilder	rowBuilder;
    protected VBoxBuilder	spanBuilder;

    /* TeXtp[786] */
    protected void	pushNewRowBuilder() {
	//XXX[786] prev_depth = 0
	rowBuilder = new VAlignBuilder(currLineNumber());
	Builder.push(rowBuilder);
    }

    /* TeXtp[787] */
    protected void	pushNewSpanBuilder() {
	spanBuilder = new VBoxBuilder(currLineNumber());
	Builder.push(spanBuilder);
    }

    protected void	addNamedSkipToRow(Glue skip, String name)
	{ rowBuilder.addNamedSkip(skip, name); }

    /* TeXtp[796] */
    protected Dimen	packedSpanSize(int spanCount) {
	NodeList	list = spanBuilder.getList();
	Builder.pop(); spanBuilder = null;
	SizesSummarizer	pack = new SizesSummarizer();
	VertIterator.summarize(list.nodes(), pack);
	Dimen	size = pack.getBody().plus(pack.getHeight())
				     .plus(pack.getDepth());
	BoxSizes	sizes = new BoxSizes(size, pack.getWidth(),
					     Dimen.ZERO, pack.getLeftX());
	byte		strOrder = pack.maxTotalStr();
	byte		shrOrder = pack.maxTotalShr();
	rowBuilder.addNode(new AnyUnsetNode(sizes, list, spanCount,
				pack.getTotalStr(strOrder), strOrder,
				pack.getTotalShr(shrOrder), shrOrder));
	return size;
    }

    /* TeXtp[799] */
    protected void	packRow() {
	NodeList	list = rowBuilder.getList();
	Builder.pop(); rowBuilder = null;
	TypoCommand.appendBox(builder,
	    new AnyUnsetNode(VertIterator.naturalSizes(list.nodes(),
						       Dimen.NULL), list));
    }

    protected NodeEnum		getUnsetNodes()
	{ return builder.getList().nodes(); }

    protected Dimen		getRelevantSize(BoxSizes sizes)
	{ return sizes.getHeight(); }

    protected BoxSizes		transformSizes(BoxSizes sizes, Dimen dim)
	{ return sizes.withHeight(dim); }

    protected AnyBoxNode	makeBox(BoxSizes sizes, GlueSetting setting,
					NodeList list)
	{ return new VBoxNode(sizes, setting, list); }

    protected AnySkipNode	makeSkip(Glue skip)
	{ return new VSkipNode(skip); }

    protected AnySkipNode	makeSkip(Glue skip, String name)
	{ return new NamedVSkipNode(skip, name); }

    protected TypoCommand.AnyBoxPacker	makeBoxPacker() {
	return new TypoCommand.VBoxPacker() {
	    /* TeXtp[675] */
	    protected void		reportLocation(Log log) {
		log.add("in alignment at lines ")
		.add(builder.getStartLine()).add("--")
		.add(currLineNumber());
	    }
	};
    }

    public void		copyPrevParameters(Builder bld) {
	bld.setSpaceFactor(builder.getSpaceFactor());
	//XXX[812] language
    }

}
