/* $Id: BoolPar.java,v 1.1.1.1 2001/05/17 23:26:40 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.base;

/** Output parameter class for the |boolean| type.
 * Used as a medium for (optional) returning values of type
 * |integer| when more return values are needed.
 *
 * @author	Karel Skoupy
 * @version	${VERSION}
 * @since	NTS1.0
 */
public class	BoolPar {

    /** Null constant for the |BoolPar| */
    public static final BoolPar		NULL = null;

    /** value of the parameter */
    private boolean	value;

    /** default constructor (default value of parameter (false)) */
    public BoolPar() { }

    /** Constructor for initial value of the parameter
     * @param	val	initial value of the parameter
     */
    public BoolPar(boolean val) { value = val; }

    /** Sets a new value of the parameter
     * @param	val	new value of the parameter
     */
    public void		set(boolean val) { value = val; }

    /** Gets the value of the parameter
     * @return	value of the parameter
     */
    public boolean	get() { return value; }

    /** Negates the value of the parameter */
    public void		negate() { value = !value; }

    /** Sets the value of the |BoolPar| if the |BoolPar| is given
     * (non-|null|)
     * @param	par	instance of |BoolPar| parameter or |null|
     * @param	val	new value of the parameter
     */
    public static void	set(BoolPar par, boolean val)
	{ if (par != NULL) par.value = val; }

}
