/* $Id: IntPairKey.java,v 1.1.1.1 2001/05/16 21:40:27 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.base;

import	java.io.Serializable;

/** Hashable pair of integers.
 * Used as a key to hash tables where association to pair of integers is
 * needed (e.g. sparse matrix).
 *
 * @author	Karel Skoupy
 * @version	${VERSION}
 * @since	NTS1.0
 */
public final class	IntPairKey	implements Serializable {

    /** first item of the pair */
    public final int		first;
    /** second item of the pair */
    public final int		second;

    /** Constructor with for a given couple of integers.
     * @param	first	first item of the pair
     * @param	second	second item of the pair
     * @return	the pair [|first|, |second|]
     */
    public IntPairKey(int first, int second)
	{ this.first = first; this.second = second; }

    /** Hash code for this |IntPairKey| object
     * @return	hash code
     */
    public int		hashCode()
	{ return 1009 * first + second; }

    /** Comparison of this object against another object.
     * @param	o	object to compare to
     * @return	|true| if |o| is an |IntPairKey| and the first resp.
     *		the second items have the same value, |false| otherwise.
     */
    public boolean		equals(Object o) {
	if (o != null && o instanceof IntPairKey) {
	    IntPairKey	k = (IntPairKey) o;
	    return (first == first && k.second == second);
	}
	return false;
    }

}
