/* $Id: HorizBuilder.java,v 1.1.1.1 2000/08/08 04:35:55 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.builder;

import	nts.io.Log;
import	nts.base.Dimen;
import	nts.base.Glue;
import	nts.node.BoxSizes;
import	nts.node.Node;
import	nts.node.NodeList;
import	nts.node.NodeEnum;
import	nts.node.Box;
import	nts.node.RuleNode;
import	nts.node.HKernNode;
import	nts.node.HSkipNode;
import	nts.node.NamedHSkipNode;
import	nts.node.Leaders;
import	nts.node.BoxLeaders;
import	nts.node.HLeadersNode;
import	nts.node.HLeadRuleNode;

public abstract class	HorizBuilder extends ListBuilder {

    public static final int	NORMAL_SPACE_FACTOR = 1000;

    private int		spaceFactor = NORMAL_SPACE_FACTOR;

    protected HorizBuilder(int line, NodeList list) { super(line, list); }
    protected HorizBuilder(int line) { super(line); }

    public boolean	isHorizontal() { return true; }
    public boolean	isCharAllowed() { return true; }

    public void		addKern(Dimen kern) { addNode(new HKernNode(kern)); }
    public void		addSkip(Glue skip) { addNode(new HSkipNode(skip)); }

    public void		addNamedSkip(Glue skip, String name)
    	{ addNode(new NamedHSkipNode(skip, name)); }

    /* TeXtp[1056] */
    public void		addRule(BoxSizes sizes)
	{ addNode(new RuleNode(sizes)); resetSpaceFactor(); }

    public void		addLeaders(Glue skip, Leaders lead)
	{ addNode(new HLeadersNode(skip, lead)); }

    public void		addLeadRule(Glue skip, BoxSizes sizes, String desc)
        { addNode(new HLeadRuleNode(skip, sizes, desc)); }

    public BoxLeaders.Mover	getBoxLeadMover()
	{ return HLeadersNode.BOX_MOVER; }

    public void		addBox(Node box)
	{ super.addBox(box); resetSpaceFactor(); }

    protected NodeEnum	unBoxList(Box box) { return box.getHorizList(); }

    /* TeXtp[1034] */
    public void		adjustSpaceFactor(int sf) {
	if (sf > 0)
	    spaceFactor = (  sf <= NORMAL_SPACE_FACTOR
	    		  || spaceFactor >= NORMAL_SPACE_FACTOR  )
			? sf : NORMAL_SPACE_FACTOR;
    }

    public int		getSpaceFactor() { return spaceFactor; }
    public int		nearestValidSpaceFactor() { return spaceFactor; }

    public void		setSpaceFactor(int sf)
	{ if (sf > 0) spaceFactor = sf; }

    public void		resetSpaceFactor()
	{ spaceFactor = NORMAL_SPACE_FACTOR; }

    /* TeXtp[219] */
    protected void	specialShow(Log log)
	{ log.startLine().add("spacefactor ").add(getSpaceFactor()); }

}
