/* $Id: ListBuilder.java,v 1.1.1.1 2000/04/12 10:43:14 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.builder;

import	nts.io.Log;
import	nts.io.CntxLog;
import	nts.node.Node;
import	nts.node.NodeList;
import	nts.node.NodeEnum;
import	nts.node.Box;

public abstract class	ListBuilder	extends Builder {

    protected NodeList		list;
    protected final int		startLine;

    protected ListBuilder(int startLine, NodeList list)
	{ this.startLine = startLine; this.list = list; }

    protected ListBuilder(int line) { this(line, new NodeList()); }

    public void		addNode(Node node) { list.append(node); }
    public void		addNodes(NodeEnum nodes) { list.append(nodes); }

    public boolean	isEmpty() { return list.isEmpty(); }
    public Node		lastNode() { return list.lastNode(); }
    public void		removeLastNode() { list.removeLastNode(); }
    public Node		lastSpecialNode() { return list.lastSpecialNode(); }

    public NodeList	getList() { return list; }
    public int		getStartLine() { return startLine; }

    protected void	specialShow(Log log, int depth, int breadth) {
	CntxLog.addItems(log, list.nodes(), depth, breadth);
	//XXX maybe there should be log.endLine()
	//XXX check everything which originates from show_box()
	specialShow(log);
    }

    protected abstract void	specialShow(Log log);

    public boolean	unBox(Box box) {
        NodeEnum	nodes = unBoxList(box);
	if (nodes != NodeEnum.NULL)
	    { addNodes(nodes); return true; }
	return false;
    }

    protected abstract NodeEnum		unBoxList(Box box);

}
