/* $Id: VertBuilder.java,v 1.1.1.1 2000/08/08 04:44:04 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.builder;

import	nts.base.Dimen;
import	nts.base.Glue;
import	nts.io.Log;
import	nts.node.Node;
import	nts.node.NodeList;
import	nts.node.NodeEnum;
import	nts.node.BoxSizes;
import	nts.node.Box;
import	nts.node.RuleNode;
import	nts.node.VKernNode;
import	nts.node.VSkipNode;
import	nts.node.NamedVSkipNode;
import	nts.node.Leaders;
import	nts.node.BoxLeaders;
import	nts.node.VLeadersNode;
import	nts.node.VLeadRuleNode;

public abstract class	VertBuilder extends ListBuilder {

    public static final Dimen	IGNORE_DEPTH = Dimen.valueOf(-1000);

    private int		prevGraf = 0;
    private Dimen	prevDepth = IGNORE_DEPTH;

    protected VertBuilder(int line, NodeList list) { super(line, list); }
    protected VertBuilder(int line) { super(line); }

    public boolean	isVertical() { return true; }
    public boolean	wantsMigrations() { return true; }
    public void		addKern(Dimen kern) { addNode(new VKernNode(kern)); }
    public void		addSkip(Glue skip) { addNode(new VSkipNode(skip)); }

    public void		addNamedSkip(Glue skip, String name)
    	{ addNode(new NamedVSkipNode(skip, name)); }

    /* TeXtp[1056] */
    public void		addRule(BoxSizes sizes) {
	addNode(new RuleNode(sizes));
	uncheckedSetPrevDepth(IGNORE_DEPTH);
    }

    public void		addLeaders(Glue skip, Leaders lead)
	{ addNode(new VLeadersNode(skip, lead)); }

    public void		addLeadRule(Glue skip, BoxSizes sizes, String desc)
        { addNode(new VLeadRuleNode(skip, sizes, desc)); }

    public BoxLeaders.Mover	getBoxLeadMover()
	{ return VLeadersNode.BOX_MOVER; }

    /* TeXtp[679] */
    public void		addBox(Node box)
	{ super.addBox(box); uncheckedSetPrevDepth(box.getDepth()); }

    protected NodeEnum	unBoxList(Box box) { return box.getVertList(); }

    public int		getPrevGraf() { return prevGraf; }
    public void		setPrevGraf(int pg) { prevGraf = pg; }

    public Dimen	getPrevDepth() { return prevDepth; }
    protected void	uncheckedSetPrevDepth(Dimen pd) { prevDepth = pd; }
    public Dimen	nearestValidPrevDepth() { return prevDepth; }

    public void		setPrevDepth(Dimen pd)
	{ if (pd != Dimen.NULL) prevDepth = pd; }

    public boolean	needsParSkip() { return (!list.isEmpty()); }

    /* TeXtp[219] */
    protected void	specialShow(Log log) {
	log.startLine().add("prevdepth ");
	if (getPrevDepth().moreThan(IGNORE_DEPTH))
	    log.add(getPrevDepth().toString());
	else log.add("ignored");
	if (prevGraf != 0)
	    log.add(", prevgraf ").add(prevGraf)
	       .add((prevGraf == 1) ? " line" : " lines");
    }

}
