/* $Id: ActiveCharToken.java,v 1.1.1.1 2000/02/16 11:00:31 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.base.EqTable;
import	nts.io.CharCode;

public class	ActiveCharToken	extends CharToken {

    public static final Maker		MAKER = new Maker() {
        public Token	make(CharCode code)
	    { return new ActiveCharToken(code); }
    };

    public ActiveCharToken(CharCode code) { super(code); }

    public interface	Meaninger {
	Command		get(CharCode code);
	void		set(CharCode code, Command cmd, boolean glob);
    }

    protected static Meaninger		meaninger;

    public static void	setMeaninger(Meaninger m) { meaninger = m; }

    /**
     * Gives Command associated in table of equivalents.
     * @return	the Command to interpret this token.
     */
    public Command	meaning() { return meaninger.get(code); }

    /**
     * Tells that the meaning of the |Token| can be redefined.
     * @return	|true|.
     */
    public boolean	definable() { return true; }

    /**
     * Define given |Command| to be equivalent in table of equivalents.
     * @param	cmd the object to interpret this token.
     * @param	glob if |true| the equivalent is defined globaly.
     */
    public void		define(Command cmd, boolean glob)
	{ meaninger.set(code, cmd, glob); }

    /**
     * Gives |CharCode.NULL| to indicate that this |Token| is an active
     * character |Token| and is not willing to provide |CharCode| upon
     * this method call.
     * @return	|CharCode.NO_CHAR|.
     */
    public CharCode	nonActiveCharCode() { return CharCode.NULL; }

    public boolean	match(CharToken tok)
	{ return (tok instanceof ActiveCharToken && tok.match(code)); }

    public Maker	getMaker() { return MAKER; }

    public String	toString() { return "<ActiveChar: " + code + '>'; }

}
