/* $Id: BackedToken.java,v 1.1.1.1 2000/08/04 14:46:32 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.base.BoolPar;
import	nts.io.Log;

/**
 * |Tokenizer| for a single |Token|.
 */
 /* See TeXtp[325]. */
public class	BackedToken	extends Tokenizer {

    /** The |Token| */
    private final Token		token;

    private boolean		readed = false;

    private final boolean	expOK;

    /**
     * Creates |Tokenizer| for a single |Token|.
     * @param	tok the single |Token|
     * @param	exp tells whether the backed |Token| can be expanded
     *		    (e.g. was not preceded by \noexpand).
     */
    public BackedToken(Token tok, boolean exp) { token = tok; expOK = exp; }

    /**
     * Gives the single |Token| on the first call.
     * @param	canExpand boolean output parameter querying whether the
     *			  acquired |Token| can be expanded (e.g. was not
     *			  preceded by \noexpand).
     * @return	the single |Token| or |Token.NULL| on next calls.
     */
    public Token	nextToken(BoolPar canExpand) {
        canExpand.set(expOK);
	if (readed) return Token.NULL;
	readed = true; return token;
    }

    public boolean	finishedList() { return readed; }

    public int		show(ContextDisplay disp, boolean force, int lines) {
        String		desc;
	Log		where;
        if (readed) {
	    if (!force) return 0;
	    desc = "<recently read> "; where = disp.left();
	} else { desc = "<to be read again> "; where = disp.right(); }
	disp.normal().startLine().add(desc);
	if (!expOK) where.addEsc("notexpanded: ");
	token.addProperlyOn(where); disp.show();
	return 1;
    }

}
