/* $Id: BaseToksChecker.java,v 1.1.1.1 1999/05/31 11:18:43 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.base.BoolPar;
import	nts.io.Loggable;

public abstract class	BaseToksChecker	implements InpTokChecker {

    protected final String		tokError;
    protected final String		eofError;
    protected final Loggable		source;

    public BaseToksChecker(String tokErr, String eofErr, Loggable src)
	{ tokError = tokErr; eofError = eofErr; source = src; }

    public Token	checkToken(Token tok, BoolPar canExpand) {
        if (Command.meaningOf(tok, canExpand.get()).isOuter()) {
	    handleOuterToken(tok);
	    reportError(tokError);
	    canExpand.set(true);
	    return SpaceToken.TOKEN;
	}
	return tok;
    }

    public void		checkEndOfFile() { reportError(eofError); }

    protected void	reportError(String ident) {
        tryToFix(); reportRunAway();
        Loggable[]	params = { source };
	Command.nonDelError(ident, params);
    }

    protected void	handleOuterToken(Token tok)
	{ Command.backTokenWithoutCleaning(tok); }

    protected abstract void	tryToFix();

    protected abstract void	reportRunAway();

}
