/* $Id: CtrlSeqToken.java,v 1.1.1.1 2000/01/28 05:50:44 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.base.EqTable;
import	nts.io.Log;
import	nts.io.Name;

/**
 * Control sequence Token.
 */
public class	CtrlSeqToken extends Token {

    public interface	Meaninger {
	Command		get(Name name);
	void		set(Name name, Command cmd, boolean glob);
    }

    protected static Meaninger		meaninger;

    public static void	setMeaninger(Meaninger m) { meaninger = m; }

    /** The name of the control sequence */
    private final Name		name;

    /**
     * Creates a control sequence token of the given name.
     * @param	name the name of the control sequence
     */
    public CtrlSeqToken(Name name) { this.name = name; }

    public CtrlSeqToken(String name) { this.name = makeName(name); }

    /**
     * Gives object (potentially) associated in table of equivalents.
     * @return	the Command to interpret this token.
     */
    public Command	meaning() { return meaninger.get(name); }

    /**
     * Tells that the meaning of the |Token| can be redefined.
     * @return	|true|.
     */
    public boolean	definable() { return true; }

    /**
     * Define given |Command| to be equivalent in table of equivalents.
     * @param	cmd the object to interpret this token.
     * @param	glob if |true| the equivalent is defined globaly.
     */
    public void		define(Command cmd, boolean glob)
	{ meaninger.set(name, cmd, glob); }

    public boolean	match(Token tok) {
        return (  tok instanceof CtrlSeqToken
	       && ((CtrlSeqToken) tok).name.match(name)  );
    }

    public void		addOn(Log log) { name.addEscapedOn(log); }

    public void		addProperlyOn(Log log)
	{ name.addProperlyEscapedOn(log); }

    public int		numValue()
	{ return (name.length() == 1) ? name.codeAt(0).numValue() : -1; }

    public Name		controlName() { return name; }

    public String	toString() { return "<CtrlSequence: " + name + '>'; }

}
