/* $Id: DefCodePrim.java,v 1.1.1.1 2000/01/28 04:58:19 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.base.Num;
import	nts.base.EqTable;
import	nts.io.Log;

/**
 */
public class	DefCodePrim	extends AssignPrim
				implements Num.Provider {

    protected final NumKind	tabKind = new NumKind();

    protected final Num		defVal;

    protected final int		maxVal;

    /**
     * Creates a new |DefCodePrim| with given name and stores it
     * in language interpreter |EqTable|.
     * @param	name the name of the |DefCodePrim|
     */
    public DefCodePrim(String name, int defVal, int maxVal) {
	super(name);
	this.defVal = (defVal != 0) ? Num.valueOf(defVal) : Num.ZERO;
	this.maxVal = maxVal;
    }

    protected final void	set(int idx, Num val, boolean glob) {
	if (glob) getEqt().gput(tabKind, idx, val);
	else getEqt().put(tabKind, idx, val);
    }

    public Num			get(int idx) {
        Num	val = (Num) getEqt().get(tabKind, idx);
	return (val != Num.NULL) ? val : defVal;
    }

    public void		addEqValueOn(int idx, Log log)
	{ log.add(get(idx).toString()); }

    /**
     * Performs the assignment.
     * @param	src source token for diagnostic output.
     * @param	glob indication that the assignment is global.
     */
    /* TeXtp[1232] */
    protected void	assign(Token src, boolean glob) {
	int	idx = scanCharacterCode();
        skipOptEquals();
	int	val = scanInt();
	if (val < 0 || val > maxVal) {
	    error("CodeOutOfRange", num(val), num(maxVal));
	    val = 0;
	}
	set(idx, Num.valueOf(val), glob);
    }

    public final boolean	hasNumValue() { return true; }
    public final Num		getNumValue()
	{ return get(scanCharacterCode()); }

    public final int		intVal(int idx) { return get(idx).intVal(); }

    public final void		init(int idx, int val)
        { set(idx, Num.valueOf(val), true); }

}
