/* $Id: FrozenToken.java,v 1.1.1.1 2000/03/03 11:29:04 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.io.Log;
import	nts.io.Name;

/**
 * Frozen control sequence Token.
 */
public class	FrozenToken extends Token {

    /** The name of the control sequence */
    private final Name		name;

    private final Command	cmd;

    /**
     * Creates a frozen control sequence token of the given name.
     * @param	name the name of the frozencontrol sequence
     * @param	cmd the meaning of the frozen control sequence
     */
    public FrozenToken(Name name, Command cmd)
	{ this.name = name; this.cmd = cmd; }

    public FrozenToken(String name, Command cmd)
	{ this.name = makeName(name); this.cmd = cmd; }

    public FrozenToken(Primitive prim) {
	this.name = makeName(prim.getName());
	this.cmd = prim.getCommand();
    }

    /**
     * Gives object (potentially) associated in table of equivalents.
     * @param	eqtab table of equivalents.
     * @return	the Command to interpret this token.
     */
    public Command	meaning() { return cmd; }

    public boolean	frozen() { return true; }

    public boolean	match(Token tok) {
    	//XXX does it match anything at all?
	//XXXX it should probably match CtrlSeqToken of the same name
	//XXXX is there any case, where it is tried to match?
	//XXXX (scanning mac params?)
        // return (  tok instanceof FrozenToken
	  //     && ((FrozenToken) tok).name.match(name)  );
	return false;
    }

    public void		addOn(Log log) { name.addEscapedOn(log); }

    public void		addProperlyOn(Log log)
	{ name.addProperlyEscapedOn(log); }

    // is the rest OK ?

    public int		numValue()
	{ return (name.length() == 1) ? name.codeAt(0).numValue() : -1; }

    public Name		controlName() { return name; }

    public String	toString() { return "<Frozen: " + name + '>'; }

}
