/* $Id: LineInputTokenizer.java,v 1.1.1.1 1999/07/28 08:07:11 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.io.InputLine;
import	nts.io.LineInput;

/**
 * The |LineInputTokenizer| reads internal character codes from
 * a |LineInput| and converts them into |Token|s.
 */
public class	LineInputTokenizer extends SequenceTokenizer {

    public interface	InputHandler {
	void		closeInput();
	InputLine	emptyLine();
	InputLine	confirmLine(InputLine line);
	Tokenizer	makeTokenizer(InputLine line, String desc,
				      boolean addEolc);
    }

    /** The underlying line oriented input reader */
    private LineInput		input;

    /** The parametrization object */
    private InputHandler	inpHandler;

    private FileName		name;

    /**
     * Creates an |LineInputTokenizer| with given input and parametrization
     * object.
     * @param	input the underlying reader.
     * @param	inpHandler the parametrization object.
     */
    public LineInputTokenizer(LineInput input, InputHandler inpHandler,
			      FileName name) {
	this.input = input;
	this.inpHandler = inpHandler;
	this.name = name;
    }

    /**
     * Closes the underlying |LineInput|.
     */
    public boolean	close()
	{ input.close(); inpHandler.closeInput(); return true; }

    private boolean	forceEOF = false;

    public boolean	endInput() { forceEOF = true; return true; }

    private boolean	used = false;
    private int		lineNum = 0;

    public Tokenizer	nextTokenizer() {
        if (forceEOF) return Tokenizer.NULL;
	InputLine	line = input.readLine();
	lineNum = input.getLineNumber();
	if (line == LineInput.EOF) {
	    //XXX maybe use parameter first of SequenceTokenizer
	    if (used) return Tokenizer.NULL;
	    line = inpHandler.emptyLine();
	    lineNum = 1;
	}
	used = true;
	line = inpHandler.confirmLine(line);
	return inpHandler.makeTokenizer(line, "l." + lineNum + ' ', true);
    }

    public boolean	enoughContext() { return true; }

    public FilePos		filePos()
	{ return new FilePos(name, lineNum); }

}
