/* $Id: MacroBody.java,v 1.1.1.1 2000/01/02 17:54:19 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.io.Log;
import	nts.io.CharCode;
import	nts.io.Name;

public class	MacroBody	extends TokenList {

    public static final MacroBody	NULL = null;

    public static final MacroBody	EMPTY = new MacroBody();

    public static class	Buffer	extends TokenList.Buffer {

	public Buffer() { super(); }
	public Buffer(int initCap) { super(initCap); }
	public Buffer(int initCap, int capIncrement)
	    { super(initCap, capIncrement); }

	public TokenList.Buffer	appendParam(int digit, CharCode matchCode)
	    { return append(new ParamToken(digit, matchCode)); }

	public MacroBody	toMacroBody() {
	    if (super.length() == 0) return EMPTY;
	    Token[]	tokens = new Token[super.length()];
	    //CCC super. required by jikes
	    data.copyInto(tokens);
	    return new MacroBody(tokens);
	}

	public TokenList	toTokenList() { return toMacroBody(); }

    }

    public MacroBody() { super(); }
    public MacroBody(Token[] tokens) { super(tokens); }
    public MacroBody(Token[] tokens, int offset, int count)
	{ super(tokens, offset, count); }

    public MacroBody(Token tok) { super(tok); }
    public MacroBody(String str) { super(str); }
    public MacroBody(Name name) { super(name); }

    private static final String		PREP = "->";
    private static final int		PLEN = PREP.length();

    public void		addOn(Log log) { log.add(PREP); super.addOn(log); }

    public void		addOn(Log log, int maxCount) {
        if (maxCount > PLEN)
	    { log.add(PREP); super.addOn(log, maxCount - PLEN); }
	else if (maxCount > 0) log.add(PREP.substring(0, maxCount));
    }

    public void		addContext(Log left, Log right, int pos, int maxCount) {
        if (maxCount > PLEN) {
	    left.add(PREP);
	    super.addContext(left, right, pos, maxCount - PLEN);
	} else if (maxCount > 0) left.add(PREP.substring(0, maxCount));
    }

}

/**
 * Token for substituting a parameter of a macro.
 */
class	ParamToken extends Token {

    /** the number of the parameter 1-9 (coded as 0..8) */
    private int		number;

    /** the character code for symbolic printing */
    private CharCode	code;

    /**
     * Creates parameter token with given parameter number.
     * @param	number the parameter number
     * @param	code the parameter character code
     */
    public ParamToken(int number, CharCode code)
	{ this.number = number; this.code = code; }

    /**
     * Tells that this |Token| is a macro parameter.
     * @return	|true|.
     */
    public boolean	isMacroParameter() { return true; }

    /**
     * Tells the number of parameter.
     * @return the macro parameter number.
     */
    public int		macroParameterNumber() { return number; }

    public boolean	match(Token tok)
	{ return (number == tok.macroParameterNumber()); }

    /**
     * Prints its symbolic representation on the Log.
     * @param   log the Log to log on
     */
    public void		addOn(Log log) {
        log.add(code);
	log.add( (0 <= number && number < 9)
	       ? Character.forDigit(number + 1, 10) : '!');
    }

}
