/* $Id: NumParam.java,v 1.1.1.1 1999/06/24 10:37:26 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.base.IntProvider;
import	nts.base.Num;
import	nts.io.Log;

/**
 * Setting number parameter primitive.
 */
public class	NumParam	extends PerformableParam
				implements IntProvider, Num.Provider {

    private Num		value;

    /**
     * Creates a new NumParam with given name and value and stores it
     * in language interpreter |EqTable|.
     * @param	name the name of the NumParam
     * @param	val the value of the NumParam
     */
    public	NumParam(String name, Num val) { super(name); value = val; }

    public	NumParam(String name, int val)
	{ this(name, Num.valueOf(val)); }

    /**
     * Creates a new NumParam with given name and stores it
     * in language interpreter |EqTable|.
     * @param	name the name of the NumParam
     */
    public	NumParam(String name) { this(name, Num.ZERO); }

    public final Object	getEqValue() { return value; }

    public final void	setEqValue(Object val) { value = (Num) val; }

    public final void	addEqValueOn(Log log) { log.add(value.toString()); }

    public Num		get() { return value; }

    public void		set(Num val, boolean glob)
	{ beforeSetting(glob); value = val; }

    public final int	intVal() { return get().intVal(); }

    protected void	scanValue(Token src, boolean glob) {
        set(scanNum(), glob);
	// System.err.println("= Assignment \\" + getName() + " = " + get());
    }

    protected void	perform(int operation, boolean glob)
	{ set(performFor(get(), operation), glob); }

    public boolean	hasNumValue() { return true; }
    public Num		getNumValue() { return get(); }

}
