/* $Id: ParamPrim.java,v 1.1.1.1 1999/06/17 11:33:12 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.base.EqTable;
import	nts.io.Log;
import	nts.io.EqTraceable;

/**
 * Abstract ancestor of each parameter primitive.
 */
public abstract class	ParamPrim extends AssignPrim
			    implements EqTable.ExtEquiv, EqTraceable {

    private int			eqLevel = 0;

    /**
     * Creates a new ParamPrim with given name and stores it
     * in language interpreter |EqTable|.
     * @param	name the name of the ParamPrim
     */
    protected	ParamPrim(String name) { super(name); }

    public final int	getEqLevel() { return eqLevel; }
    public final void	setEqLevel(int lev) { eqLevel = lev; }
    public final void	addEqDescOn(Log log) { log.addEsc(getName()); }
    public final void	retainEqValue() { traceRestore(RETAINING, this); }

    public final void	restoreEqValue(Object val)
	{ setEqValue(val); traceRestore(RESTORING, this); }

    public abstract Object	getEqValue();
    public abstract void	setEqValue(Object val);
    public abstract void	addEqValueOn(Log log);

    /**
     * Performs the assignment.
     * @param	src source token for diagnostic output.
     * @param	glob indication that the assignment is global.
     */
    protected final void	assign(Token src, boolean glob) {
        skipOptEquals();
	scanValue(src, glob);
    }

    protected abstract void	scanValue(Token src, boolean glob);

    protected final void	beforeSetting(boolean glob)
	{ getEqt().beforeSetting(this, glob); }

}
