/* $Id: PrefixPrim.java,v 1.1.1.1 2001/03/22 13:34:20 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.io.Log;
import	nts.io.Name;

/**
 * Prim command of the macro language which can be prefixed
 * by one of prefix commands (as \global, \long or \outer).
 */
public abstract class	PrefixPrim extends Prim {

    public static final int	LONG = 1;
    public static final int	OUTER = 2;
    public static final int	GLOBAL = 4;

    /** inaccessible but definable token */
    private static final Token	INACCESSIBLE = new Token() {
        private final Name	NAME = makeName("inaccessible");
        public boolean	definable() { return true; }
	public void	define(Command cmd, boolean glob) { }
	public boolean	match(Token tok) { return false; }
	public void	addOn(Log log) { NAME.addEscapedOn(log); }
	public void	addProperlyOn(Log log)
	    { NAME.addProperlyEscapedOn(log); }
    };

    /**
     * Creates a new PrefixPrim with given name and stores it
     * in language interpreter |EqTable|.
     * @param	name the name of the PrefixPrim
     */
    protected	PrefixPrim(String name) { super(name); }

    /** Non prefixed version of exec */
    public final void	exec(Token src) { exec(src, 0); }

    public abstract void	exec(Token src, int prefixes);
    public final boolean	assignable() { return true; }
    public void			doAssignment(Token src, int prefixes)
				    { exec(src, prefixes); }

    /* TeXtp[1215] */
    public static Token		definableToken() {
	for (;;) {
	    Token		tok;
	    while ((tok = nextRawToken()).matchSpace());
	    if (tok.definable()) return tok;
	    else {
		if (!tok.frozen()) backToken(tok);
		insertToken(INACCESSIBLE);
		error("MissingCtrlSeq");
	    }
	}
    }

    public static final int	BOOLP_ALWAYS_GLOBAL = newBoolParam();
    public static final int	BOOLP_NEVER_GLOBAL = newBoolParam();

    public static boolean	globalAssignment(int prefixes) {
	return (  getConfig().getBoolParam(BOOLP_ALWAYS_GLOBAL)
	       || !getConfig().getBoolParam(BOOLP_NEVER_GLOBAL)
	       && (prefixes & GLOBAL) != 0  );
    }

    public static void		beforeAssignment(Command cmd, int prefixes) {
        if ((prefixes & (LONG | OUTER)) != 0)
	    error("NonDefineCommand", esc("long"), esc("outer"), cmd);
    }

    public static void		afterAssignment()
	{ getConfig().afterAssignment(); }

}
