/* $Id: SequenceTokenizer.java,v 1.1.1.1 1999/05/31 11:18:51 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.base.BoolPar;

public abstract class	SequenceTokenizer extends Tokenizer {

    /** The current input line */
    private Tokenizer	curr;

    public SequenceTokenizer() { curr = Tokenizer.NULL; }

    public SequenceTokenizer(Tokenizer first) { curr = first; }

    /**
     * Gets the next |Token| from the input.
     * @param	canExpand boolean output parameter querying whether the
     *			  acquired |Token| can be expanded (e.g. was not
     *			  preceded by \noexpand).
     * @return	the next token from the input or |Token.NULL| if the input
     *		is finished.
     */
    public Token	nextToken(BoolPar canExpand) {
        if (curr == Tokenizer.NULL) curr = nextTokenizer();
	while (curr != Tokenizer.NULL) {
	    Token	tok = curr.nextToken(canExpand);
	    if (tok != Token.NULL) return tok;
	    curr = nextTokenizer();
	}
	return Token.NULL;
    }

    public boolean	enoughContext() {
        // if (curr == Tokenizer.NULL) curr = nextTokenizer();
	return (curr != Tokenizer.NULL) ? curr.enoughContext() : false;
    }

    public int		show(ContextDisplay disp, boolean force, int lines) {
        // if (curr == Tokenizer.NULL) curr = nextTokenizer();
	return (curr != Tokenizer.NULL) ? curr.show(disp, force, lines) : 0;
    }

    public abstract Tokenizer	nextTokenizer();

}
