/* $Id: ShorthandDefPrim.java,v 1.1.1.1 2001/03/22 13:35:01 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.io.Log;

public abstract class	ShorthandDefPrim	extends AssignPrim {

    public ShorthandDefPrim(String name) { super(name); }

    protected void	assign(Token src, boolean glob) {
	Token	tok = definableToken();
	tok.define(Relax.getRelax(), glob);
	skipOptEquals();
	int	idx = scanRegisterCode();
	tok.define(makeShorthand(idx), glob);
    }

    protected abstract Command	makeShorthand(int idx);

    public static abstract class	Shorthand	extends Command {

	protected final int		index;

	public Shorthand(int idx) { index = idx; }

	/** Non prefixed version of exec */
	public final void	exec(Token src) { exec(src, 0); }

	public final boolean	assignable() { return true; }
	public final void	doAssignment(Token src, int prefixes)
				    { exec(src, prefixes); }

	/**
	 * Performs itself in the process of interpretation of the macro language
	 * after sequence of prefix commands.
	 * @param	src source token for diagnostic output.
	 * @param	prefixes accumulated code of prefixes.
	 */
	public final void		exec(Token src, int prefixes) {
	    PrefixPrim.beforeAssignment(this, prefixes);
	    assign(src, PrefixPrim.globalAssignment(prefixes));
	    PrefixPrim.afterAssignment();
	}

	protected abstract void	assign(Token src, boolean glob);

	public final void		addOn(Log log)
	    { log.addEsc(getReg().getEqDesc()).add(index); }

	public boolean		sameAs(Command cmd) {
	    return (  getClass() == cmd.getClass()
	    	   && getReg().sameAs(((Shorthand) cmd).getReg())
		   && index == ((Shorthand) cmd).index  );
	}

	protected abstract RegisterPrim		getReg();

    }

}
