/* $Id: SpaceToken.java,v 1.1.1.1 2000/02/16 11:01:14 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.io.Log;
import	nts.io.CharCode;

public class	SpaceToken extends Token {

    public static final CharCode	CODE = CharToken.makeCharCode(' ');
    public static final SpaceToken	TOKEN = new SpaceToken();

    public CharCode	nonActiveCharCode() { return CODE; }

    public int		numValue() { return CODE.numValue(); }

    public CharCode	charCode() { return CODE; }

    public Token	makeCharToken(CharCode code)
	{ return DirtySpaceToken.MAKER.make(code); }

    public boolean	matchSpace() { return true; }

    public boolean	match(Token tok)
	{ return (tok instanceof SpaceToken); }

    public boolean	sameCatAs(Token tok) {
        return (  tok instanceof SpaceToken
	       || tok instanceof DirtySpaceToken  );
    }

    public void		addOn(Log log) { log.add(' '); }

    public String	toString() { return "<Space>"; }

    private static CharHandler		handler;

    public static void		setHandler(CharHandler hnd)
	{ handler = hnd; DirtySpaceToken.setHandler(hnd); }

    public static final String		DESCRIPTION = "blank space";

    public static final Command		MEANING = new Command() {

	public final boolean	isSpacer() { return true; }

	public CharCode		charCode() { return CODE; }

	public void	exec(Token src) { handler.handle(CODE, src); }

	public final void	exec(Token src, int prefixes) {
	    Token		tok = nextExpToken();
	    meaningOf(tok).execute(tok, prefixes);
	}

	public void		addOn(Log log)
	    { log.add(DESCRIPTION).add(' ').add(' '); }

	public boolean		sameAs(Command cmd)
	    { return (getClass() == cmd.getClass()); }

	public Token		origin() { return TOKEN; }

    };

    public Command	meaning() { return MEANING; }

}

class	DirtySpaceToken extends CharToken {

    public static final Maker		MAKER = new Maker() {
        public Token	make(CharCode code) {
	    if (code.match(' ')) return SpaceToken.TOKEN;
	    return new DirtySpaceToken(code);
	}
    };

    public DirtySpaceToken(CharCode code) { super(code); }

    public boolean	match(CharToken tok)
	{ return (tok instanceof DirtySpaceToken && tok.match(code)); }

    public boolean	sameCatAs(Token tok)
	{ return SpaceToken.TOKEN.sameCatAs(tok); }

    public Maker	getMaker() { return MAKER; }

    public String	toString() { return "<DirtySpace: " + code + '>'; }

    private static CharHandler		handler;

    public static void		setHandler(CharHandler hnd) { handler = hnd; }

    public Command	meaning() {
	return new Meaning() {

	    public void		exec(Token src)
		{ handler.handle(code, src); }

	    public final void	exec(Token src, int prefixes) {
		Token		tok = nextExpToken();
		meaningOf(tok).execute(tok, prefixes);
	    }

	    protected String	description()
		{ return SpaceToken.DESCRIPTION; }

	};
    }

}
