/* $Id: TokenListTokenizer.java,v 1.1.1.1 2000/08/04 14:46:43 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.base.BoolPar;

/**
 * |Tokenizer| for sequence of |Token|s in an array.
 */
 /* See TeXtp[323]. */
public abstract class	TokenListTokenizer	extends Tokenizer {

    /** Array of |Token|s where the sequence is stored */
    private final TokenList	list;

    /** The position of the next |Token| in the list */
    private int			pos;

    /** Position after the last |Token| in the sequence */
    private final int		end;

    /**
     * Creates |Tokenizer| for a portion of a |TokenList|.
     * @param	list the |TokenList|.
     * @param	start position of the first |Token| in the sequence.
     * @param	end position after the last |Token| in the sequence.
     */
    public TokenListTokenizer(TokenList list, int start, int end)
	{ this.list = list; this.pos = start; this.end = end; }

    /**
     * Creates |Tokenizer| for a |TokenList|.
     * @param	list the |TokenList|.
     */
    public TokenListTokenizer(TokenList list)
	{ this.list = list; this.pos = 0; this.end = list.length(); }

    /**
     * Gives the next |Token| from the sequence.
     * @param	canExpand boolean output parameter querying whether the
     *			  acquired |Token| can be expanded (e.g. was not
     *			  preceded by \noexpand).
     * @return	next |Token| or |Token.NULL| when the sequence is finished.
     */
    public Token	nextToken(BoolPar canExpand) {
        canExpand.set(true);
	return (pos < end) ? list.tokenAt(pos++) : Token.NULL;
    }

    public boolean	finishedList() { return (pos >= end); }

    public int	showList(ContextDisplay disp, int lines) {
	list.addContext(disp.left(), disp.right(), pos, 100000);
	disp.show();			//XXX why so much?
	return 1;
    }

}
