/* $Id: Tokenizer.java,v 1.1.1.1 2000/01/27 16:54:19 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.command;

import	nts.base.BoolPar;

/**
 * Base abstract class for all |Tokenizer|s.
 */
public abstract class	Tokenizer {

    public static final Tokenizer	NULL = null;

    /** The stack where this instance is pushed to */
    private TokenizerStack	stack = null;

    /** The next |Tokenizer| in the stack (the one under this one) */
    private Tokenizer		next = NULL;

    /**
     * Gets the stack of |Tokenizer|s where this |Tokenizer| is pushed.
     * @return	the stack which this |Tokenizer| belongs to or |null|
     *		if this |Tokenizer| is not pushed.
     */
    public TokenizerStack	getStack() { return stack; }

    public Tokenizer	getNext() { return next; }

    synchronized void	pushTo(TokenizerStack stack, Tokenizer next) {
	if (this.stack != null)
	    throw new RuntimeException("Tokenizer is already in a stack");
	if (this != stack.getTop())	//SSS
	    throw new RuntimeException("Misused Tokenizer.pushTo()");
	this.stack = stack;
	this.next = next;
    }

    synchronized void	popFrom(TokenizerStack stack) {
	if (this.stack != stack || this.next != stack.getTop())	//SSS
	    throw new RuntimeException("Misused Tokenizer.popFrom()");
	this.stack = null;
	this.next = null;
    }

    /**
     * Gets the next |Token| from this |Tokenizer|.
     * @param	canExpand boolean output parameter querying whether
     *			  the acquired |Token| can be expanded
     *			  (e.g. was not preceded by \noexpand).
     * @return	the next |Token| or |Token.NULL| if this |Tokenizer|
     *		is finished.
     */
    public abstract Token	nextToken(BoolPar canExpand);

    /**
     * Should be overriden by a subclass in order to release resources
     * used by this |Tokenizer|.
     */
    public boolean	close() { return false; }

    public boolean	finishedList() { return false; }

    public boolean	finishedInsert() { return false; }

    public boolean	finished() { return finishedList(); }

    public boolean	endInput() { return false; }

    public boolean	enoughContext() { return false; }

    public FilePos	filePos() { return FilePos.NULL; }

    public abstract int	show(ContextDisplay disp, boolean force, int lines);

}
