/* $Id: DviOutputStream.java,v 1.1.1.1 1999/09/03 17:16:55 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.dvi;

import	java.io.IOException;
import	java.io.OutputStream;
import	java.io.FilterOutputStream;

public class	DviOutputStream	extends FilterOutputStream {

    private static final int	DEFAULT_BUF_SIZE = 0x4000;

    private final int			half;
    private final byte[]		buffer;

    private int			offset = 0;
    private int			gone = 0;
    private int			ptr = 0;
    private int			limit;

    public DviOutputStream(OutputStream out, int size) {
        super(out);
	if (size <= 0) size = DEFAULT_BUF_SIZE;
	size = size + 7 & ~7; half = size / 2;
	buffer = new byte[size]; limit = size;
    }

    public DviOutputStream(OutputStream out) { this(out, 0); }

    /* STRANGE
     * Buffer could be flushed on next write leaving larger space for set().
     */
    /* TeXtp[598] */
    public void		write(int b) throws IOException {
        buffer[ptr++] = (byte) b;
	if (ptr == limit) {
	    if (limit == half) limit = buffer.length;
	    else { offset += ptr; ptr = 0; limit = half; }
	    out.write(buffer, ptr, half);
	    gone += half;
	}
    }

    /* TeXtp[599] */
    public void		flush() throws IOException {
        if (limit == half)
	    { out.write(buffer, half, half); gone += half; }
	if (ptr > 0) {
	    out.write(buffer, 0, ptr); gone += ptr;
	    offset += ptr; ptr = 0;
	}
	out.flush(); limit = buffer.length;
    }

    /* STRANGE
     * It could be droped almost always
     * (except after flush() and just after starting).
     */
    public boolean	dropLast() {
        if (ptr > 0) { ptr--; return true; }
	else return false;
    }

    public int		pos() { return offset + ptr; }

    public boolean	present(int pos)
	{ return (pos >= gone && pos < offset + ptr); }

    public int		get(int pos) {
        if (pos >= gone && (pos -= offset) < ptr) {
	    if (pos < 0) pos += buffer.length;
	    return (buffer[pos] & 0xff);
	}
	return -1;
    }

    public boolean	set(int pos, int b) {
        if (pos >= gone && (pos -= offset) < ptr) {
	    if (pos < 0) pos += buffer.length;
	    buffer[pos] = (byte) b; return true;
	}
	return false;
    }

}
