/* $Id: BufferLineOutput.java,v 1.1.1.1 2001/02/03 13:13:46 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.io;

public final class	BufferLineOutput	extends BaseLineOutput
			implements Loggable, CharCode.CodeWriter {

    private CharCode		buffer[];
    private int			maxCount;
    private int			pos;
    private CharCode.Maker	maker;

    public BufferLineOutput(int size, int maxCount,
			    CharCode.Maker maker) {
	buffer = new CharCode[size];
	this.maxCount = maxCount;
	this.maker = maker; pos = 0;
    }

    public void		add(CharCode code)
	{ code.writeExpCodes(this); }

    public void		add(char chr)
	{ maker.writeExpCodes(chr, this); }

    public void		writeCode(CharCode code) {
	if (maxCount <= 0 || pos < maxCount)
	    buffer[pos++ % buffer.length] = code;
	charCount++;
    }

    public void		reset() {
	for (int i = 0; i < buffer.length; buffer[i++] = CharCode.NULL);
	pos = 0; resetCount();
    }

    public int		size()
	{ return (pos <= buffer.length) ? pos : buffer.length; }

    public void		addOn(Log log, int idx)
        { log.add(codeAt(idx)); }

    public void		addOn(Log log, int beg, int end)
	{ while (beg < end) log.add(codeAt(beg++)); }

    public void		addOn(Log log) { addOn(log, 0, size()); }

    private CharCode	codeAt(int idx) {
	if (idx >= 0) {
	    if (pos <= buffer.length)
		{ if (idx < pos) return buffer[idx]; }
	    else if (idx < buffer.length)
		return buffer[(pos + idx) % buffer.length];
	}
	throw new ArrayIndexOutOfBoundsException(
	    "BufferLineOutput: index out of range: " + idx);
    }

}
