/* $Id: CntxLog.java,v 1.1.1.1 2000/05/02 06:02:30 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.io;

public final class	CntxLog {

    private final int		maxDepth;
    private final int		maxCount;
    private final String	prefix;

    private CntxLog(int md, int mc, String p)
	{ maxDepth = md; maxCount = mc; prefix = p; }

    private CntxLog(int md, int mc)
	{ maxDepth = md; maxCount = mc; prefix = ""; }

    private int		count = 0;

    /* STRANGE
     * this method is public (and count is not a local variable)
     * only for third part of discretionary
     */
    /* TeXtp[182] */
    public void	addItems(Log log, CntxLoggableEnum items) {
	if (prefix.length() <= maxDepth) {
	    while (count <= maxCount && items.hasMoreContextLoggables()) {
		log.endLine().add(prefix);
		if (count++ == maxCount) log.add("etc.");
		else items.nextContextLoggable().addOn(log, this);
	    }
	} else if (count <= maxCount && items.hasMoreContextLoggables())
	    log.add(" []");
    }

    private void	addItem(Log log, CntxLoggable item) {
	if (prefix.length() <= maxDepth) {
	    if (count <= maxCount) {
		log.endLine().add(prefix);
		if (count++ == maxCount) log.add("etc.");
		else item.addOn(log, this);
	    }
	} else if (count <= maxCount)
	    log.add(" []");
    }

    private CntxLog		descendant(char p)
	{ return new CntxLog(maxDepth, maxCount, prefix + p); }

    public void		addOn(Log log, CntxLoggableEnum items, char p)
	{ descendant(p).addItems(log, items); }

    public void		addOn(Log log, CntxLoggable item, char p)
	{ descendant(p).addItem(log, item); }

    public void		addOn(Log log, CntxLoggableEnum items)
	{ addOn(log, items, '.'); }

    public void		addOn(Log log, CntxLoggable item)
	{ addOn(log, item, '.'); }

    public static void		addItems(Log log, CntxLoggableEnum items,
					 int md, int mc)
	{ (new CntxLog(md, mc)).addItems(log, items); }

    public static void		addItem(Log log, CntxLoggable item,
					int md, int mc)
	{ (new CntxLog(md, mc)).addItem(log, item); }

}
