/* $Id: LineInput.java,v 1.1.1.1 1999/04/20 21:10:23 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.io;

import	java.io.Reader;
import	java.io.LineNumberReader;
import	java.io.IOException;

/**
 * |LineInput| reads the input line by line and performs the conversions from
 * input character codes to their internal representation.
 */
public class	LineInput {

    /** Value returned by |readLine| when the input is finished */
    public static final InputLine	EOF = InputLine.NULL;

    /** Underlying reader */
    private LineNumberReader		in;

    /** Input character mapper */
    private InputLine.Mapper		mapper;

    private boolean			trouble = false;

    /**
     * Creates |LineInput| with given |LineNumberReader| and |InputLine.Mapper|.
     * @param	in the LineNumberReader to read from.
     * @param	mapper the mapper for character conversions etc.
     */
    public LineInput(LineNumberReader in, InputLine.Mapper mapper)
        { this.in = in; this.mapper = mapper; }

    /**
     * Creates |LineInput| with given |Reader| and |InputLine.Mapper|.
     * @param	in the Reader to read from.
     * @param	mapper the mapper for character conversions etc.
     */
    public LineInput(Reader in, InputLine.Mapper mapper)
        { this(new LineNumberReader(in), mapper); }

    /**
     * Releases the system resources associated with the |LineInput|.
     */
    public void		close() {
        try { in.close(); }
	catch (IOException e) { trouble = true; }
    }

    /**
     * Reads one line from the input, remove trailing characters which should
     * be ignored (usually spaces) and appends end line character if defined.
     * @param	addEolc flag indicating whether the |endLineChar| (if active)
     *		should be added.
     * @return	an object representing the current input line
     *		or |EOF| if the input is finished.
     */
    public InputLine	readLine() {
	String		str;
	try { str = in.readLine(); }
	catch (IOException e) {
	    str = null; trouble = true;
	    System.err.println("I/O error: " + e);
	}
	return (str != null) ? new InputLine(str, mapper) : EOF;
    }

    public int		getLineNumber() { return in.getLineNumber(); }

}
