/* $Id: StandardLog.java,v 1.1.1.1 2001/02/06 12:37:07 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.io;

/**
 * Object which performs printing to the terminal and log file.
 */
public class	StandardLog	implements Log {

    public interface	Escape { CharCode	getEscape(); }

    protected final LineOutput		out;
    protected final Escape		esc;

    public StandardLog(LineOutput out, Escape esc)
	{ this.out = out; this.esc = esc; }

    public final LineOutput	getOutput() { return out; }
    public final Escape		getEscape() { return esc; }

    public final Log	add(char ch) { out.add(ch); return this; }
    public final Log	add(char ch, int count)
	{ while (count-- > 0) out.add(ch); return this; }
    public final Log	add(String str) { out.add(str); return this; }
    public final Log	add(CharCode code) { out.add(code); return this; }
    public final Log	endLine() { out.endLine(); return this; }
    public final Log	startLine() { out.startLine(); return this; }
    public final Log	flush() { out.flush(); return this; }
    public final void	close() { out.close(); }

    public final Log	add(boolean val) { return add(String.valueOf(val)); }
    public final Log	add(int num) { return add(String.valueOf(num)); }

    public final Log	addEsc() {
        CharCode	escape = esc.getEscape();
	return (escape != CharCode.NULL) ? add(escape) : this;
    }

    public final Log	addEsc(String str) { addEsc(); return add(str); }

    public final Log	add(Loggable x) { x.addOn(this); return this; }
    public Log		add(Loggable[] array)
	{ return add(array, 0, array.length); }
    public Log		add(Loggable[] array, int offset, int length) {
	for (int i = 0; i < length; add(array[offset + i++]));
	return this;
    }

    public final Log	resetCount() { out.resetCount(); return this; }
    public final int	getCount() { return out.getCount(); }
    public final Log	voidCounter()
	{ return new StandardLog(out.voidCounter(), esc); }

    public final Log	sepRoom(int count) {
	if (!out.stillFits(count)) out.endLine();
	else if (!out.isStartLine()) out.add(' ');
	return this;
    }

}
