/* $Id: WriterLineOutput.java,v 1.1.1.1 2001/02/03 13:22:24 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.io;

import	java.io.Writer;
import	java.io.IOException;

public final class	WriterLineOutput	extends BaseLineOutput
					implements CharCode.CharWriter {

    private Writer		out;
    private CharCode.Maker	maker;
    private boolean		worryRoom;
    private int			maxLineChars;
    private int			numLineChars = 0;
    private String		lineSeparator;
    private boolean		trouble = false;

    public WriterLineOutput(Writer out, CharCode.Maker maker,
    			    boolean worryRoom, int maxLineChars) {
	this.out = out;
	this.maker = maker;
	this.worryRoom = (worryRoom && maxLineChars > 0);
	this.maxLineChars = maxLineChars;
	lineSeparator = System.getProperty("line.separator");
    }

    public WriterLineOutput(Writer out, CharCode.Maker maker)
	{ this(out, maker, false, 0); }

    public final void		add(CharCode code) {
        if (code.isNewLine()) endLine();
	else code.writeExpChars(this);
    }

    public final void		add(char chr) {
        if (maker.isNewLine(chr)) endLine();
	else maker.writeExpChars(chr, this);
    }

    public final void		writeChar(char chr) {
	try {
	    out.write(chr); charCount++; numLineChars++;
	    if (maxLineChars > 0 && numLineChars >= maxLineChars) endLine();
	} catch (IOException x) { trouble = true; }
    }

    public final void		addRaw(String str) {
	try { out.write(str); }
	catch (IOException x) { trouble = true; }
    }

    public final void		endLine() {
        try { out.write(lineSeparator); out.flush(); }
	catch (IOException x) { trouble = true; }
	numLineChars = 0;
    }

    public final void		flush() {
        try { out.flush(); }
	catch (IOException x) { trouble = true; }
    }

    public final void		close() {
        try { out.close(); }
	catch (IOException x) { trouble = true; }
    }

    public final void		setStartLine() { numLineChars = 0; }

    public final boolean	isStartLine() { return (numLineChars == 0); }

    public final boolean	stillFits(int count)
	{ return !(worryRoom && numLineChars + count > maxLineChars); }

}
