/* $Id: FamilyPrim.java,v 1.1.1.1 2001/02/01 16:20:20 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.math;

import	nts.base.EqTable;
import	nts.io.Log;
import	nts.node.FontMetric;
import	nts.command.Token;
import	nts.command.CtrlSeqToken;
import	nts.command.TokenList;
import	nts.command.AssignPrim;
import	nts.typo.TypoCommand;
import	nts.typo.NullFontMetric;

public class	FamilyPrim	extends AssignPrim
				implements TokenList.Provider {

    protected final NumKind	tabKind = new NumKind();

    protected final FontMetric	defVal;

    public FamilyPrim(String name, FontMetric defVal)
	{ super(name); this.defVal = defVal; }

    public FamilyPrim(String name)
	{ super(name); this.defVal = NullFontMetric.METRIC; }

    protected final void	set(int idx, FontMetric val, boolean glob) {
	if (glob) getEqt().gput(tabKind, idx, val);
	else getEqt().put(tabKind, idx, val);
    }

    public final FontMetric	get(int idx) {
        FontMetric	val = (FontMetric) getEqt().get(tabKind, idx);
	return (val != FontMetric.NULL) ? val : defVal;
    }

    public void		addEqValueOn(int idx, Log log)
        { get(idx).getIdent().addEscapedOn(log); }

    /**
     * Performs the assignment.
     * @param	src source token for diagnostic output.
     * @param	glob indication that the assignment is global.
     */
    /* TeXtp[1234] */
    protected void	assign(Token src, boolean glob) {
	int	idx = scanFamilyCode();
        skipOptEquals();
	set(idx, TypoCommand.scanFontMetric(), glob);
    }

    public boolean	hasFontTokenValue() { return true; }
    public boolean	hasFontMetricValue() { return true; }

    /* TeXtp[415,465] */
    public Token	getFontTokenValue()
	{ return new CtrlSeqToken(get(scanFamilyCode()).getIdent()); }

    /* TeXtp[577] */
    public FontMetric	getFontMetricValue()
	{ return get(scanFamilyCode()); }

    public final void		init(int idx, FontMetric val)
        { set(idx, val, true); }

    public static int	scanFamilyCode() { return scanFileCode(); }

}
