/* $Id: OpenMath.java,v 1.1.1.1 2001/03/12 21:48:59 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.math;

import	nts.base.Dimen;
import	nts.io.Log;
import	nts.builder.Builder;
import	nts.node.NodeList;
import	nts.node.LinesShape;
import	nts.node.FontMetric;
import	nts.command.Token;
import	nts.command.TokenList;
import	nts.typo.Action;
import	nts.typo.BuilderCommand;
import	nts.typo.Paragraph;

public class	OpenMath	extends BuilderCommand {

    private /* final */ TokenList.Inserter	everyMath;
    private /* final */ TokenList.Inserter	everyDisplay;

    public OpenMath(TokenList.Inserter everyMath,
		    TokenList.Inserter everyDisplay) {
	this.everyMath = everyMath;
	this.everyDisplay = everyDisplay;
    }

    /* TeXtp[1138] */
    public final Action		OPEN_MATH = new Action() {
	public void		exec(Builder bld, Token src) {
	    Token	tok = nextRawToken();
	    if (meaningOf(tok).isMathShift()) startDisplay();
	    else { backToken(tok); startFormula(); }
	}
    };

    /* STRANGE
     * Why is next token read when it is not needed at all?
     */
    /* TeXtp[1138] */
    public final Action		OPEN_FORMULA = new Action() {
	public void		exec(Builder bld, Token src)
	    { backToken(nextRawToken()); startFormula(); }
    };

    public static final int	INTP_DISPLAY_WIDOW_PENALTY = newIntParam();
    private static final Dimen	NO_WIDTH = Dimen.MAX_VALUE.negative();

    protected void	startDisplay() {
	Dimen.Par	lastVisibleWidth = new Dimen.Par(NO_WIDTH);
	Builder		parBld = getBld();
	NodeList	list = parBld.getParagraph();
	if (list != NodeList.NULL)
	    Paragraph.lineBreak(list, parBld.getStartLine(),
			getConfig().getIntParam(INTP_DISPLAY_WIDOW_PENALTY),
				parBld.getInitLang(), lastVisibleWidth);
	Dimen		preSize = lastVisibleWidth.get();
	if (preSize == Dimen.NULL) preSize = Dimen.MAX_VALUE;
	else if (preSize != NO_WIDTH)
	    preSize = preSize.plus(getCurrFontMetric().getDimenParam(
				    FontMetric.DIMEN_PARAM_QUAD).times(2));
	LinesShape	shape = getTypoConfig().linesShape();
	Builder		old = getBld();
	int		lineNo = old.getPrevGraf() + 1;
	pushLevel(new DisplayGroup());
	MathPrim.getMathConfig().initDisplay(preSize, shape.getWidth(lineNo),
					     shape.getIndent(lineNo));
	everyDisplay.insertToks();
	old.buildPage();	//XXX is this order unavoidable?
    }

    protected void	startFormula() {
	pushLevel(new FormulaGroup());
	MathPrim.getMathConfig().initFormula();
	everyMath.insertToks();
    }

    public void		addOn(Log log) { log.add("internal open math"); }

}
