/* $Id: Delimiter.java,v 1.1.1.1 2000/04/08 11:54:34 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.noad;

import	nts.io.Log;
import	nts.io.Loggable;
import	nts.io.CharCode;

public class	Delimiter	implements Loggable {

    public static final Delimiter	NULL = null;
    public static final Delimiter	VOID
	= new Delimiter((byte) 0, CharCode.NULL, (byte) 0, CharCode.NULL);

    private final byte		smallFam;
    private final CharCode	smallCode;
    private final byte		largeFam;
    private final CharCode	largeCode;

    public Delimiter(byte smallFam, CharCode smallCode,
		     byte largeFam, CharCode largeCode) {
	this.smallFam = smallFam; this.smallCode = smallCode;
	this.largeFam = largeFam; this.largeCode = largeCode;
    }

    public byte		getSmallFam() { return 	smallFam; }
    public CharCode	getSmallCode() { return smallCode; }
    public byte		getLargeFam() { return 	largeFam; }
    public CharCode	getLargeCode() { return largeCode; }

    public boolean	isVoid() {
	return (  smallFam == 0 && smallCode == CharCode.NULL
	       && largeFam == 0 && largeCode == CharCode.NULL  );
    }

    /* STRANGE
     * why not print mnemonic values?
     */
    /* TeXtp[691] */
    public void		addOn(Log log) {
	int	n = (famCharNum(smallFam, smallCode) << 12)
		   + famCharNum(largeFam, largeCode);
	log.add('"').add(Integer.toHexString(n).toUpperCase());
    }

    private int		famCharNum(byte fam, CharCode code) {
	int	n = fam << 8;
	if (code != CharCode.NULL) n += code.numValue();
	return n;
    }

}
