/* $Id: MathNode.java,v 1.1.1.1 2000/05/26 21:16:46 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.noad;

import	nts.base.Dimen;
import	nts.io.Log;
import	nts.io.CntxLog;
import	nts.node.Node;
import	nts.node.NodeList;
import	nts.node.NodeEnum;
import	nts.node.FontMetric;
import	nts.node.BreakingCntx;
import	nts.node.DiscardableNode;

public abstract class	MathNode	extends DiscardableNode {
    /* root corresponding to math_node */

    protected final Dimen		kern;

    public MathNode(Dimen kern) { this.kern = kern; }

    public Dimen	getKern() { return kern; }
    public Dimen	getWidth() { return kern; }

    /* TeXtp[192] */
    public void		addOn(Log log, CntxLog cntx) {
	log.addEsc((isOn()) ? "mathon" : "mathoff");
	if (!kern.isZero()) log.add(", surrounded ").add(kern.toString());
    }

    public FontMetric	addShortlyOn(Log log, FontMetric metric)
	{ log.add('$'); return metric; }

    public void		addBreakDescOn(Log log) { log.addEsc("math"); }
    public boolean	isKernBreak() { return true; }

    public int		breakPenalty(BreakingCntx brCntx)
	{ return (brCntx.spaceBreaking()) ? 0 : INF_PENALTY; }

    public NodeEnum	atBreakReplacement()
	{ return NodeList.nodes(resizedCopy(Dimen.ZERO)); }

    protected abstract boolean	isOn();
    protected abstract MathNode	resizedCopy(Dimen kern);

}
