/* $Id: OperatorEgg.java,v 1.1.1.1 2000/10/16 11:03:27 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.noad;

import	nts.base.Dimen;
import	nts.node.Node;
import	nts.node.Box;
import	nts.node.HBoxNode;
import	nts.node.HShiftNode;

public class	OperatorEgg	extends Egg {

    protected final Node	node;
    protected final Dimen	shift;
    protected final byte	spType;

    /* STRANGE
     * node might be null only if the metric or char was not available.
     * In that case it becomes empty hbox like in case of epmty field but
     * that hbox is surprisingly shifted -- see TeXtp[749].
     * It is the only reason for all tests (node == Node.NULL).
     */

    public OperatorEgg(Node node, Dimen shift, byte spType)
	{ this.node = node; this.shift = shift; this.spType = spType; }

    private boolean	suppressed = false;

    public void		suppressItalCorr() { suppressed = true; }

    public Dimen	getItalCorr() {
	return (suppressed || node == Node.NULL) ? Dimen.NULL
						 : node.getItalCorr();
    }

    public byte		spacingType() { return spType; }

    public Dimen	getHeight() {
	return (node == Node.NULL) ? Dimen.ZERO
	     : node.getHeight().max(Dimen.ZERO).minus(shift);
    }

    public Dimen	getDepth() {
	return (node == Node.NULL) ? Dimen.ZERO
	     : node.getDepth().max(Dimen.ZERO).plus(shift);
    }

    public void		chipShell(Nodery nodery) {
	Box		box = (node == Node.NULL) ? HBoxNode.EMPTY
						  : HBoxNode.packedOf(node);
	Dimen		ital = getItalCorr();
	if (ital != Dimen.NULL)
	    box = box.pretendingWidth(box.getWidth().plus(ital));
	nodery.append(HShiftNode.shiftingDown(box, shift));
    }

    public boolean	isPenalty()
	{ return (node != Node.NULL && node.isPenalty()); }

}
