/* $Id: PureNoad.java,v 1.1.1.1 2000/08/09 06:15:49 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.noad;

import	nts.base.Dimen;
import	nts.node.GlueSetting;
import	nts.node.Node;
import	nts.node.NodeList;
import	nts.node.BoxSizes;
import	nts.node.RuleNode;
import	nts.node.IntVKernNode;
import	nts.node.HShiftNode;
import	nts.node.VBoxNode;
import	nts.node.SizesSummarizer;
import	nts.node.VertIterator;

public abstract class	PureNoad	extends BaseNoad {

    public final boolean	isNode() { return false; }
    public Node			getNode() { return Node.NULL; }

    /* TeXtp[704] */
    public static Node		makeRule(Dimen height) {
	return new RuleNode(new BoxSizes(height, Dimen.NULL,
					 Dimen.ZERO, Dimen.NULL));
    }

    /* TeXtp[705] */
    public static Node		makeOverBar(Node node, Dimen clearence,
					    Dimen thickness, Dimen extra) {
	NodeList	list = new NodeList(4);
	list.append(new IntVKernNode(extra));
	list.append(makeRule(thickness));
	list.append(new IntVKernNode(clearence));
	list.append(node);
	return VBoxNode.packedOf(list);
    }

    /* TeXtp[735] */
    public static Node		makeUnderBar(Node node, Dimen clearence,
					     Dimen thickness, Dimen extra) {
	NodeList	list = new NodeList(3);
	list.append(node);
	list.append(new IntVKernNode(clearence));
	list.append(makeRule(thickness));
	SizesSummarizer		pack = new SizesSummarizer();
	VertIterator.summarize(list.nodes(), pack);
	BoxSizes		sizes = new BoxSizes(
			    pack.getHeight(), pack.getWidth(),
			    pack.getBody().plus(pack.getDepth()).plus(extra),
			    pack.getLeftX());
	return new VBoxNode(sizes, GlueSetting.NATURAL, list);
    }

    /* TeXtp[706] */
    public static Node	varDelimiter(Delimiter del, Dimen size,
				     Transformer transf) {
	Node		node = transf.fetchSufficientNode(del, size);
	Dimen		shift
	    = node.getHeight().minus(node.getDepth()).halved()
	     .minus(transf.getDimPar(DP_AXIS_HEIGHT));
	return HShiftNode.shiftingDown(node, shift);
    }

}
