/* $Id: RadicalNoad.java,v 1.1.1.1 2000/04/15 18:04:20 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.noad;

import	nts.base.Dimen;
import	nts.io.Log;
import	nts.io.CntxLog;
import	nts.node.Node;
import	nts.node.NodeList;
import	nts.node.HBoxNode;
import	nts.node.HShiftNode;

public class	RadicalNoad	extends ScriptableNoad {

    protected final Delimiter		radical;
    protected final Field		nucleus;

    public RadicalNoad(Delimiter radical, Field nucleus)
	{ this.radical = radical; this.nucleus = nucleus; }

    /* TeXtp[696] */
    public void		addOn(Log log, CntxLog cntx) {
	log.addEsc(getDesc()).add(radical);
	nucleus.addOn(log, cntx, '.');
    }

    protected String	getDesc() { return "radical"; }
    protected byte	spacingType() { return SPACING_TYPE_ORD; }

    public Egg		convert(Converter conv) {
	Node	node = nucleus.cleanBox(conv, CRAMPED);
	Dimen	drt = conv.getDimPar(DP_DEFAULT_RULE_THICKNESS);
	Dimen	clr = drt.plus(( (conv.getStyle() != DISPLAY_STYLE) ? drt
			       : conv.getDimPar(DP_MATH_X_HEIGHT))
			       .absolute().over(4));
	Dimen	size = node.getHeight().plus(node.getDepth()).plus(clr);
	Node	rad = conv.fetchSufficientNode(radical, size.plus(drt));
	Dimen	height = rad.getHeight();
	size = rad.getDepth().minus(size);
	if (size.moreThan(0)) clr = clr.plus(size.halved());
	rad = HShiftNode.shiftingUp(rad, node.getHeight().plus(clr));
	NodeList	list = new NodeList(2);
	list.append(rad).append(makeOverBar(node, clr, height, height));
	return new StSimpleNodeEgg(HBoxNode.packedOf(list), spacingType());
    }

}
