/* $Id: VCenterNoad.java,v 1.1.1.1 2001/03/22 15:44:37 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.noad;

import	nts.base.Dimen;
import	nts.io.Log;
import	nts.io.CntxLog;
import	nts.node.Node;
import	nts.node.Box;
import	nts.node.BoxSizes;
import	nts.node.HShiftNode;

public class	VCenterNoad	extends ScriptableNoad {

    protected final NodeField		nucleus;

    public VCenterNoad(NodeField nucleus) { this.nucleus = nucleus; }

    /* TeXtp[696] */
    public void		addOn(Log log, CntxLog cntx) {
	log.addEsc(getDesc());
	nucleus.addOn(log, cntx, '.');
    }

    protected String	getDesc() { return "vcenter"; }

    public boolean	isJustChar() { return nucleus.isJustChar(); }
    //XXX would it be true (for script drops) even if the result of nucleus
    //XXX conversion was really just char?

    /* STRANGE
     * why \vcenter lays about box sizes instead of simply shifting it?
     */
    /* TeXtp[736] */
    public Egg		convert(Converter conv) {
	Node		node = nucleus.getNode();
	Dimen		middle = conv.getDimPar(DP_AXIS_HEIGHT);
	if (node.isBox()) {
	    Box		box = node.getBox();
	    BoxSizes	sizes = box.getSizes();
	    Dimen	shift = delta(sizes.getHeight(),
				      sizes.getDepth(), middle);
	    node = box.pretendSizesCopy(sizes.shiftedUp(shift));
	} else {
	    Dimen	shift = delta(node.getHeight(),
				      node.getDepth(), middle);
	    node = HShiftNode.shiftingUp(node, shift);
	}
	return new StItalNodeEgg(node, SPACING_TYPE_ORD);
    }

    private Dimen	delta(Dimen height, Dimen depth, Dimen middle) {
	return depth.minus(height).halved().plus(middle);
	// Dimen		total = height.plus(depth);
	// return total.halved().plus(middle).minus(height);
    }

}
