/* $Id: AnyBoxNode.java,v 1.1.1.1 2001/04/27 14:40:21 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.node;

import	nts.base.Dimen;
import	nts.io.Log;
import	nts.io.CntxLog;

public abstract class	AnyBoxNode	extends AnyBoxedNode
					implements Box {
    /* corresponding to hlist_node, vlist_node */

    protected final GlueSetting		setting;
    protected NodeList			list;

    public AnyBoxNode(BoxSizes sizes, GlueSetting setting, NodeList list)
	{ super(sizes); this.setting = setting; this.list = list; }

    public GlueSetting	getSetting() { return setting; }
    public NodeList	getList() { return list; }

    public boolean	isVoid() { return false; }
    public boolean	isHBox() { return false; }
    public boolean	isVBox() { return false; }

    public NodeEnum	getHorizList()
	{ return (isHBox()) ? list.nodes(): NodeEnum.NULL; }

    public NodeEnum	getVertList()
	{ return (isVBox()) ? list.nodes(): NodeEnum.NULL; }

    public boolean	isBox() { return true; }
    public Box		getBox() { return this; }
    public boolean	isCleanBox() { return true; }
    public boolean	canBePartOfDiscretionary() { return true; }
    protected boolean	allegedlyVisible() { return true; }

    public Box		pretendingWidth(Dimen width) {
	return (getWidth().equals(width)) ? this
	     : pretendSizesCopy(sizes.withWidth(width));
    }

    public void		typeSet(TypeSetter setter)
	{ typeSet(setter, true); }

    public void		typeSet(TypeSetter setter, SettingContext sctx) {
	if (!list.isEmpty())
	    { setter.push(); typeSet(setter, sctx.allowIO); setter.pop(); }
    }

    private void	typeSet(TypeSetter setter, boolean allowIO) {
        TypeSetter.Mark		here = setter.mark();
	NodeEnum		nodes = list.nodes();
        moveStart(setter);
	SettingContext		sctx
	    = new SettingContext(sizes, setting, setter.mark(), allowIO);
	while (nodes.hasMoreNodes()) {
	    Node	node = nodes.nextNode();
	    movePrev(setter, node);
	    node.typeSet(setter, sctx);
	    movePast(setter, node);
	}
	here.move();
    }

    public void		syncVertIfBox(TypeSetter setter)
	{  if (!list.isEmpty()) setter.syncVert(); }

    protected abstract void	moveStart(TypeSetter setter);
    protected abstract void	movePrev(TypeSetter setter, Node node);
    protected abstract void	movePast(TypeSetter setter, Node node);

    public void		addOn(Log log, CntxLog cntx, Dimen shift) {
	log.addEsc(getDesc()).add(sizes).add(setting);
	if (shift != Dimen.NULL && !shift.isZero())
	    log.add(", shifted ").add(shift.toString());
	cntx.addOn(log, list.nodes());
    }

    public void		addOn(Log log, CntxLog cntx)
	{ addOn(log, cntx, Dimen.NULL); }

    public void		addOn(Log log, int maxDepth, int maxCount)
	{ CntxLog.addItem(log, this, maxDepth, maxCount); }

    public void		addListShortlyOn(Log log) { list.addShortlyOn(log); }

    public abstract String	getDesc();

    /* TeXtp[715] */
    public Node		reboxedToWidth(Dimen width) {
	return (list.isEmpty()) ? pretendingWidth(width)
	     : super.reboxedToWidth(width);
    }

}
