/* $Id: BaseFontMetric.java,v 1.1.1.1 2001/02/09 12:53:15 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.node;

import	nts.base.Num;
import	nts.base.Dimen;
import	nts.base.Glue;

public abstract class	BaseFontMetric	implements FontMetric {

    public boolean	isNull() { return false; }

    private Num[]		numPars = new Num[NUMBER_OF_NUM_PARAMS];
    { for (int i = 0; i < numPars.length; numPars[i++] = Num.NULL); }

    public Num	getNumParam(int idx) {
	return (0 <= idx && idx < numPars.length)
	     ? numPars[idx] : Num.NULL;
    }

    public Num	setNumParam(int idx, Num val) {
	return (0 <= idx && idx < numPars.length)
	     ? (numPars[idx] = val) : Num.NULL;
    }

    public boolean	definesNumParams(int[] idxs) {
	for (int i = 0; i < idxs.length; i++) {
	    int		idx = idxs[i];
	    if (  idx < 0 || idx >= numPars.length
	       || numPars[idx] == Num.NULL  ) return false;
	}
	return true;
    }

    /*
     * We make no assumptions about the order of DIMEN_PARAM constants
     * therefore we simply allocate the array for all possible parameters and
     * initialize each element to Dimen.NULL. Subclasses of this class can
     * assign appropriate values to those parameters which they really provide
     * during construction.
     */

    private Dimen[]		dimPars = new Dimen[NUMBER_OF_DIMEN_PARAMS];
    {
	for (int i = 0; i < dimPars.length; dimPars[i++] = Dimen.NULL);
	final int[]	params = ALL_TEXT_DIMEN_PARAMS;
	for (int i = 0; i < params.length; dimPars[params[i++]] = Dimen.ZERO);
    }

    public Dimen	getDimenParam(int idx) {
	return (0 <= idx && idx < dimPars.length)
	     ? dimPars[idx] : Dimen.NULL;
    }

    public Dimen	setDimenParam(int idx, Dimen val) {
	if (0 <= idx && idx < dimPars.length) {
	    normalSpace = Glue.NULL;
	    return dimPars[idx] = val;
	}
	return Dimen.NULL;
    }

    public boolean	definesDimenParams(int[] idxs) {
	for (int i = 0; i < idxs.length; i++) {
	    int		idx = idxs[i];
	    if (  idx < 0 || idx >= dimPars.length
	       || dimPars[idx] == Dimen.NULL  ) return false;
	}
	return true;
    }

    private transient Glue	normalSpace = Glue.NULL;

    public Glue		getNormalSpace() {
        if (normalSpace == Glue.NULL)
	    normalSpace = Glue.valueOf( dimPars[DIMEN_PARAM_SPACE],
					dimPars[DIMEN_PARAM_STRETCH],
					dimPars[DIMEN_PARAM_SHRINK] );
	return normalSpace;
    }

}
