/* $Id: BoxLeaders.java,v 1.1.1.1 2000/02/19 01:42:33 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.node;

import	java.io.Serializable;
import	nts.base.Dimen;
import	nts.base.Glue;
import	nts.io.Log;
import	nts.io.CntxLog;

public abstract class	BoxLeaders	implements Leaders {

    public interface	Mover	extends Serializable {
        Dimen		offset(TypeSetter.Mark start);
        Dimen		size(Node node);
	void		move(TypeSetter setter, Dimen gap);
        void		movePrev(TypeSetter setter, Node node);
	void		movePast(TypeSetter setter, Node node);
    }

    public static final Mover	NULL_MOVER = null;

    protected final Node	node;
    protected final Mover	mover;

    public BoxLeaders(Node node, Mover mover)
	{ this.node = node; this.mover = mover; }

    public Dimen	getHeight() { return node.getHeight(); }
    public Dimen	getWidth() { return node.getWidth(); }
    public Dimen	getDepth() { return node.getDepth(); }
    public Dimen	getLeftX() { return node.getLeftX(); }

    /* TeXtp[190] */
    public void		addOn(Log log, CntxLog cntx, Glue skip) {
	log.addEsc(getDesc()).add(' ').add(skip.toString());
	cntx.addOn(log, node);
    }

    private static final Dimen	compensation = Dimen.valueOf(10, 0x10000);

    public void		typeSet(TypeSetter setter, Dimen size,
    				SettingContext sctx) {
	if (size.moreThan(0)) {
	    Dimen	nodeSize = mover.size(node);
	    if (nodeSize.moreThan(0))
	        typeSet(setter, sctx.allowingIO(false),
			size.plus(compensation), nodeSize);
	}
    }

    /* TeXtp[626,628,635,637] */
    protected void	typeSet(TypeSetter setter, SettingContext sctx,
    				int count, Dimen start, Dimen gap) {
	mover.move(setter, start);
	while (count-- > 0) {
	    mover.movePrev(setter, node);
	    node.typeSet(setter, sctx);
	    mover.movePast(setter, node);
	    mover.move(setter, gap);
	}
    }

    protected abstract void	typeSet(TypeSetter setter, SettingContext sctx,
    					Dimen size, Dimen nodeSize);

    protected abstract String	getDesc();

}
