/* $Id: BoxSizes.java,v 1.1.1.1 2000/04/13 09:22:05 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.node;

import	java.io.Serializable;
import	nts.base.Dimen;
import	nts.io.Log;
import	nts.io.Loggable;

public final class	BoxSizes	implements Serializable, Loggable {

    public static final BoxSizes	NULL = null;

    public static final BoxSizes	ZERO
	= new BoxSizes(Dimen.ZERO, Dimen.ZERO, Dimen.ZERO, Dimen.ZERO);

    private final Dimen		height;
    private final Dimen		width;
    private final Dimen		depth;
    private final Dimen		leftX;

    public BoxSizes(Dimen h, Dimen w, Dimen d, Dimen l)
	{ height = h; width = w; depth = d; leftX = l; }

    public Dimen	rawHeight() { return height; }
    public Dimen	rawWidth() { return width; }
    public Dimen	rawDepth() { return depth; }
    public Dimen	rawLeftX() { return leftX; }

    public Dimen	getHeight() { return getSize(height); }
    public Dimen	getWidth() { return getSize(width); }
    public Dimen	getDepth() { return getSize(depth); }
    public Dimen	getLeftX() { return getSize(leftX); }

    private static Dimen	getSize(Dimen x)
	{ return (x != Dimen.NULL) ? x : Dimen.ZERO; }

    public BoxSizes	replenished(BoxSizes around) {
        Dimen		h = height;
	Dimen		w = width;
        Dimen		d = depth;
        Dimen		l = leftX;
	boolean		change = false;
	if (h == Dimen.NULL) { h = around.getHeight(); change = true; }
	if (w == Dimen.NULL) { w = around.getWidth(); change = true; }
	if (d == Dimen.NULL) { d = around.getDepth(); change = true; }
	if (l == Dimen.NULL) { l = around.getLeftX(); change = true; }
	return (change) ? new BoxSizes(h, w, d, l) : this;
    }

    public BoxSizes	shiftedUp(Dimen shift) {
	return new BoxSizes(
	    (height != Dimen.NULL) ? height.plus(shift) : height, width,
	    (depth != Dimen.NULL) ? depth.minus(shift) : depth, leftX);
    }

    public BoxSizes	shiftedLeft(Dimen shift) {
	return new BoxSizes(
	    height, (width != Dimen.NULL) ? width.minus(shift) : width,
	    depth, (leftX != Dimen.NULL) ? leftX.plus(shift) : leftX);
    }

    public BoxSizes	withHeight(Dimen height)
	{ return new BoxSizes(height, width, depth, leftX); }

    public BoxSizes	withWidth(Dimen width)
	{ return new BoxSizes(height, width, depth, leftX); }

    public BoxSizes	withDepth(Dimen depth)
	{ return new BoxSizes(height, width, depth, leftX); }

    public BoxSizes	withLeftX(Dimen leftX)
	{ return new BoxSizes(height, width, depth, leftX); }

    public void		addOn(Log log) {
	log.add('('); addSize(log, height);
	log.add('+'); addSize(log, depth);
	log.add(")x"); addSize(log, width);
    }

    private static void		addSize(Log log, Dimen x) {
        if (x == Dimen.NULL) log.add('*');
	else log.add(x.toString());
    }

}
