/* $Id: GlueSetting.java,v 1.1.1.1 2001/05/15 03:58:40 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.node;

import	java.io.Serializable;
import	nts.base.Dimen;
import	nts.base.Glue;
import	nts.io.Log;
import	nts.io.Loggable;

public class	GlueSetting	implements Serializable, Loggable {

    public static final byte
    	RIGID		= 0,
	STRETCHING	= 1,
	SHRINKING	= 2;

    protected final byte		sign;
    protected final byte		order;
    protected final double		ratio;

    public static final GlueSetting	NATURAL
	= new GlueSetting(RIGID, Glue.NORMAL, 0.0);

    public GlueSetting(byte sign, byte order, double ratio)
	{ this.sign = sign; this.order = order; this.ratio = ratio; }

    /* TeXtp[625, 634] */
    public Dimen	set(Glue glue, boolean limited) {
	Dimen		dimen = glue.getDimen();
	switch (sign) {
	    case STRETCHING:
	        if (glue.getStrOrder() == order)
		    dimen = (limited)
			  ? dimen.plus(glue.getStretch().limitedTimes(ratio))
			  : dimen.plus(glue.getStretch().times(ratio));
		break;
	    case SHRINKING:
	        if (glue.getShrOrder() == order)
		    dimen = (limited)
			  ? dimen.minus(glue.getShrink().limitedTimes(ratio))
			  : dimen.minus(glue.getShrink().times(ratio));
		break;
	}
	return dimen;
    }

    /* TeXtp[1148] */
    public boolean	makesElastic(Glue glue) {
	switch (sign) {
	    case STRETCHING:
	        return (  glue.getStrOrder() == order
		       && !glue.getStretch().isZero()  );
	    case SHRINKING:
	        return (  glue.getShrOrder() == order
		       && !glue.getShrink().isZero()  );
	}
	return false;
    }

    public void		addOn(Log log) {
	if (sign != RIGID && ratio != 0.0) {
	    log.add(", glue set ");
	    if (sign == SHRINKING) log.add("- ");
	    if (Math.abs(ratio) > Dimen.MAX_DOUBLE_VALUE)
	        log.add((ratio > 0.0) ? ">" : "< -")
		   .add(Glue.toString(Dimen.MAX_FROM_DOUBLE, order));
	    else log.add(Glue.toString(Dimen.valueOf(ratio), order));
	}
    }

}
