/* $Id: HangIndent.java,v 1.1.1.1 1999/11/16 17:55:52 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.node;

import	nts.base.Dimen;

public class	HangIndent	implements LinesShape {

    private final int		boundary;
    private final Dimen		firstWidth;
    private final Dimen		firstIndent;
    private final Dimen		secondWidth;
    private final Dimen		secondIndent;

    public HangIndent(int b, Dimen fw, Dimen fi, Dimen sw, Dimen si) {
	boundary = b;
	firstWidth = fw; firstIndent = fi;
	secondWidth = sw; secondIndent = si;
    }

    public HangIndent(Dimen sw, Dimen si)
	{ this(0, Dimen.ZERO, Dimen.ZERO, sw, si); }

    public boolean	isFinal(int idx) { return (idx >= boundary); }

    public Dimen	getWidth(int idx)
	{ return (idx < boundary) ?  firstWidth : secondWidth; }

    public Dimen	getIndent(int idx)
	{ return (idx < boundary) ?  firstIndent : secondIndent; }

    /* TeXtp[849] */
    public static LinesShape	makeShape(int ha, Dimen hi, Dimen hs) {
	if (hi.isZero()) return new HangIndent(hs, Dimen.ZERO);
	Dimen		size, ind;
	if (hi.lessThan(0)) { ind = Dimen.ZERO; size = hs.plus(hi); }
	else { ind = hi; size = hs.minus(hi); }
	return (ha < 0) ? new HangIndent(-ha, size, ind, hs, Dimen.ZERO)
			: new HangIndent(ha, hs, Dimen.ZERO, size, ind);
    }

}
