/* $Id: Language.java,v 1.1.1.1 2000/10/18 21:45:31 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.node;

import	java.io.Serializable;
import	nts.io.Log;
import	nts.io.Loggable;

public abstract class	Language	implements Serializable, Loggable {

    public static final Language	NULL = null;
    public static final char		WORD_BOUNDARY = 0;

    protected final int		languageNumber;
    protected final int		leftHyphenMin;
    protected final int		rightHyphenMin;

    public Language(int ln, int lhm, int rhm) {
	languageNumber = ln;
	leftHyphenMin = lhm;
	rightHyphenMin = rhm;
    }

    public boolean	isCommon() {
        return (  languageNumber == 0
	       && leftHyphenMin == 2
	       && rightHyphenMin == 3  );
    }

    /* TeXtp[218] */
    public void		addOn(Log log) {
	log.add("language").add(languageNumber)
	   .add(":hyphenmin").add(leftHyphenMin)
	   .add(',').add(rightHyphenMin);
    }

    /* STRANGE
     * Why two so similar forms?
     */
    /* TeXtp[1356] */
    public void		addShortlyOn(Log log) {
	log.add(languageNumber)
	   .add(" (hyphenmin ").add(leftHyphenMin)
	   .add(',').add(rightHyphenMin).add(')');
    }

    public boolean	isZero() { return (languageNumber == 0); }
    public void		addNumberOn(Log log) { log.add(languageNumber); }

    public boolean	sameNumberAs(int number)
	{ return (languageNumber == number); }

    public abstract Hyphens	getHyphens(String word);
    public abstract void	setHyphException(String word, int[] positions);
    public abstract boolean	setHyphPattern(String pattern, int[] values);

}
