/* $Id: NetDimen.java,v 1.1.1.1 2000/01/28 17:17:18 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.node;

import	java.io.Serializable;
import	nts.base.Dimen;
import	nts.base.Glue;

public class	NetDimen	implements Serializable {

    public static final NetDimen	NULL = null;

    private Dimen	natural;
    private Dimen	shrink;
    private Dimen[]	stretch = new Dimen[Glue.MAX_ORDER + 1];

    public Dimen	getNatural() { return natural; }
    public Dimen	getShrink() { return shrink; }
    public Dimen	getStretch(byte ord) { return stretch[ord]; }

    public void		setNatural(Dimen dim) { natural = dim; }
    public void		setShrink(Dimen dim) { shrink = dim; }
    public void		setStretch(Dimen dim, byte ord) { stretch[ord] = dim; }

    public byte		getMaxStrOrder() {
	byte		ord = Glue.MAX_ORDER;
	while (ord > Glue.NORMAL && stretch[ord].isZero()) ord--;
	return ord;
    }

    public NetDimen() { this(Dimen.ZERO); }

    public NetDimen(Dimen d) {
	natural = d; shrink = Dimen.ZERO;
	for (int i = 0; i < stretch.length; stretch[i++] = Dimen.ZERO);
    }

    public NetDimen(Glue g) {
	natural = g.getDimen();
	shrink = g.getShrink();
	for (int i = 0; i < stretch.length; stretch[i++] = Dimen.ZERO);
	stretch[g.getStrOrder()] = g.getStretch();
    }

    public NetDimen(NetDimen nd) {
	natural = nd.natural;
	shrink = nd.shrink;
	for (int i = 0; i < stretch.length; i++)
	    stretch[i] = nd.stretch[i];
    }

    public final void	add(Dimen d) { natural = natural.plus(d); }
    public final void	addShrink(Dimen d) { shrink = shrink.plus(d); }

    public final void	addStretch(byte ord, Dimen d)
	{ stretch[ord] = stretch[ord].plus(d); }

    public void		add(Glue g) {
	add(g.getDimen()); addShrink(g.getShrink());
	addStretch(g.getStrOrder(), g.getStretch());
    }

    public void		add(NetDimen nd) {
	add(nd.natural); addShrink(nd.shrink);
	for (byte o = 0; o < stretch.length; o++)
	    addStretch(o, nd.stretch[o]);
    }

    public final void	sub(Dimen d) { natural = natural.minus(d); }
    public final void	subShrink(Dimen d) { shrink = shrink.minus(d); }

    public final void	subStretch(byte ord, Dimen d)
	{ stretch[ord] = stretch[ord].minus(d); }

    public void		sub(Glue g) {
	sub(g.getDimen()); subShrink(g.getShrink());
	subStretch(g.getStrOrder(), g.getStretch());
    }

    public void		sub(NetDimen nd) {
	sub(nd.natural); subShrink(nd.shrink);
	for (byte o = 0; o < stretch.length; o++)
	    subStretch(o, nd.stretch[o]);
    }

    public String	toString() { return toString(null); }

    /* TeXtp[985] */
    public String	toString(String unit) {
        StringBuffer	buf = new StringBuffer(128);
	buf.append(natural.toString());
	if (unit != null) buf.append(unit);
	for (byte o = 0; o < stretch.length; o++)
	    if (!stretch[o].isZero())
		Glue.append(buf.append(" plus "), stretch[o], o, unit);
	if (!shrink.isZero())
	    Glue.append(buf.append(" minus "), shrink, Glue.NORMAL, unit);
	return buf.toString();
    }

}
