/* $Id: Node.java,v 1.1.1.1 2000/06/06 11:51:31 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.node;

import	java.io.Serializable;
import	nts.base.Num;
import	nts.base.Dimen;
import	nts.base.Glue;
import	nts.io.Log;
import	nts.io.Name;
import	nts.io.CntxLog;
import	nts.io.CntxLoggable;
import	nts.command.TokenList;	//DDD

public interface	Node	extends Serializable, CntxLoggable {
    /* corresponding to ANY_node */

    Node	NULL = null;
    int		INF_PENALTY = Dimen.INF_BAD;
    int		EJECT_PENALTY = -INF_PENALTY;

    public interface	Provider { Node	getNodeValue(); }
    //CCC jikes requires public here

    Dimen		getHeight();
    Dimen		getDepth();
    Dimen		getHstr();
    byte		getHstrOrd();
    Dimen		getHshr();
    byte		getHshrOrd();

    Dimen		getWidth();
    Dimen		getLeftX();
    Dimen		getWstr();
    byte		getWstrOrd();
    Dimen		getWshr();
    byte		getWshrOrd();

    boolean		sizeIgnored();
    boolean		isPenalty();
    boolean		isKern();
    boolean		hasKern();		/* STRANGE : only for [424] */
    boolean		isSkip();
    boolean		isMuSkip();
    boolean		isBox();
    boolean		isCleanBox();		/* STRANGE : only for [720] */
    boolean		isMigrating();
    boolean		isMark();
    boolean		isInsertion();

    Num			getPenalty();
    Dimen		getKern();
    Glue		getSkip();
    Glue		getMuSkip();
    Box			getBox();
    Dimen		getItalCorr();
    NodeEnum		getMigration();
    TokenList		getMark();
    Insertion		getInsertion();

    void		typeSet(TypeSetter setter, SettingContext sctx);
    FontMetric		addShortlyOn(Log log, FontMetric metric);
    void		addOn(Log log, CntxLog cntx);
    void		addOn(Log log, CntxLog cntx, Dimen shift);
    void		syncVertIfBox(TypeSetter setter);

    Dimen		getHeight(GlueSetting setting);
    Dimen		getDepth(GlueSetting setting);
    Dimen		getWidth(GlueSetting setting);
    Dimen		getLeftX(GlueSetting setting);

    void		addBreakDescOn(Log log);
    boolean		discardable();
    boolean		isKernBreak();
    boolean		canPrecedeSkipBreak();
    boolean		canFollowKernBreak();
    boolean		allowsSpaceBreaking();
    boolean		forbidsSpaceBreaking();
    int			breakPenalty(BreakingCntx brCntx);
    boolean		isHyphenBreak();
    Dimen		preBreakWidth();
    Dimen		postBreakWidth();
    NodeEnum		atBreakReplacement();
    NodeEnum		postBreakNodes();
    boolean		discardsAfter();
    boolean		canBePartOfDiscretionary();

    void		contributeVisible(VisibleSummarizer summarizer);
    boolean		kernAfterCanBeSpared();
    boolean		isKernThatCanBeSpared();
    Node		trailingKernSpared();
    Node		reboxedToWidth(Dimen width);

    byte	FAILURE = 0;
    byte	SKIP = 1;
    byte	SUCCESS = 2;

    boolean		startsWordBlock();
    byte		beforeWord();
    boolean		canBePartOfWord();
    FontMetric		uniformMetric();
    Language		alteringLanguage();
    void		contributeCharCodes(Name.Buffer buf);
    byte		afterWord();

    boolean		rightBoundary();
    boolean		providesRebuilder(boolean prev);
    WordRebuilder	makeRebuilder(TreatNode proc, boolean prev);

}
