/* $Id: SizesEvaluator.java,v 1.1.1.1 2000/10/25 07:16:30 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.node;

import	nts.base.Dimen;
import	nts.base.Glue;

public class	SizesEvaluator	extends SizesSummarizer {

    public static final int	OK = 0;
    public static final int	UNDERFULL = 1;
    public static final int	TIGHT = 2;
    public static final int	OVERFULL = 3;

    protected int		badness;
    protected int		report;
    protected Dimen		overfull;
    protected GlueSetting	setting;

    public int		getBadness() { return badness; }
    public int		getReport() { return report; }
    public Dimen	getOverfull() { return overfull; }
    public GlueSetting	getSetting() { return setting; }

    /* TeXtp[668] */
    public void		evaluate(Dimen excess, boolean empty) {
	badness = 0; report = OK; overfull = Dimen.ZERO;
	byte		sign = GlueSetting.RIGID;
	byte		order = Glue.NORMAL;
	double		ratio = 0.0;
	if (!excess.isZero()) {
	    byte		trySign;
	    Dimen		total;
	    if (excess.moreThan(0)) {
		trySign = GlueSetting.STRETCHING;
		order = maxTotalStr();
		total = getTotalStr(order);
	    } else {
		trySign = GlueSetting.SHRINKING;
		order = maxTotalShr();
		total = getTotalShr(order);
	        excess = excess.negative();
	    }
	    if (!total.isZero())
		{ ratio = excess.doubleOver(total); sign = trySign; }
	    if (!empty && order == Glue.NORMAL) {
		if (trySign == GlueSetting.STRETCHING)
		    { report = UNDERFULL; badness = excess.badness(total); }
		else if (excess.moreThan(total)) {
		    report = OVERFULL; badness = Dimen.OVERFULL_BAD;
		    ratio = 1.0; overfull = excess.minus(total);
		} else { report = TIGHT; badness = excess.badness(total); }
	    }
	}
	setting = new GlueSetting(sign, order, ratio);
    }

}
