/* $Id: SizesSummarizer.java,v 1.1.1.1 2000/08/09 08:28:31 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.node;

import	nts.base.Dimen;
import	nts.base.Glue;

public class	SizesSummarizer {

    private Dimen	height;
    private Dimen	body;
    private Dimen	depth;
    private Dimen	width;
    private Dimen	leftX;
    private Dimen[]	totalStr = new Dimen[Glue.MAX_ORDER + 1];
    private Dimen[]	totalShr = new Dimen[Glue.MAX_ORDER + 1];

    public final Dimen	getHeight() { return height; }
    public final Dimen	getBody() { return body; }
    public final Dimen	getDepth() { return depth; }
    public final Dimen	getWidth() { return width; }
    public final Dimen	getLeftX() { return leftX; }
    public final Dimen	getTotalStr(byte ord) { return totalStr[ord]; }
    public final Dimen	getTotalShr(byte ord) { return totalShr[ord]; }
    public final byte	maxTotalStr() { return maxOrd(totalStr); }
    public final byte	maxTotalShr() { return maxOrd(totalShr); }

    public SizesSummarizer() {
	height = Dimen.ZERO; body = Dimen.ZERO; depth = Dimen.ZERO;
	width = Dimen.ZERO; leftX = Dimen.ZERO;
	setZero(totalStr); setZero(totalShr);
    }

    public final void	add(Dimen dim) { body = body.plus(dim); }
    public final void	setHeight(Dimen h) { add(height); height = h; }
    public final void	setDepth(Dimen d) { add(depth); depth = d; }
    public final void	setMaxWidth(Dimen w) { width = width.max(w); }
    public final void	setMaxLeftX(Dimen l) { leftX = leftX.max(l); }

    public final void	addStretch(Dimen dim, byte ord)
	{ addOrd(totalStr, dim, ord); }

    public final void	addShrink(Dimen dim, byte ord)
	{ addOrd(totalShr, dim, ord); }

    public final void	add(Glue glue) {
	body = body.plus(glue.getDimen());
	addStretch(glue.getStretch(), glue.getStrOrder());
	addShrink(glue.getShrink(), glue.getShrOrder());
    }

    public final void	restrictDepth(Dimen maxDepth) {
	if (depth.moreThan(maxDepth))
	    { body = body.plus(depth).minus(maxDepth); depth = maxDepth; }
    }

    private void	setZero(Dimen[] total)
	{ for (int i = total.length; i-- > 0; total[i] = Dimen.ZERO); }

    private void	addOrd(Dimen[] total, Dimen dim, byte ord)
	{ if (!dim.isZero()) total[ord] = total[ord].plus(dim); }

    private byte	maxOrd(Dimen[] total) {
        byte		ord = Glue.MAX_ORDER;
	while (ord > Glue.NORMAL && total[ord].isZero()) ord--;
	return ord;
    }

}
