/* $Id: PLDumper.java,v 1.1.1.1 1999/07/28 08:07:15 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.tfm;

/**
 * Interface |PLDumper| defines methods which class should provide
 * to produce a property list.
 *
 * Each property list consist of property name folowed by zero or more
 * property values. The values in turn may be property lists too.
 * For detailed description of property list see the documentation of pltotf
 * program.
 *
 * The interface is abstract in the sense that it makes no asumption about
 * output formatting or media.
 *
 * Each of output methods returns reference to PLDumper (typicaly |this|
 * reference). It allows for chaining several output steps together.
 * For example: |dumper.open("NUMBER").addDec(number).close();|
 */
public interface	PLDumper {

    /**
     * Opens a new property list with given property name.
     * @param	s the name of the property to be listed.
     * @return	|PLDumper| for further output.
     */
    PLDumper	open(String p);

    /**
     * Finishes the current list.
     * @return	|PLDumper| for further output.
     */
    PLDumper	close();

    /**
     * Add a decimal number as a property value.
     * @param	i the number to be added to current list.
     * @return	|PLDumper| for further output.
     */
    PLDumper	addDec(int i);

    /**
     * Add an octal number as a property value.
     * @param	i the number to be added to current list.
     * @return	|PLDumper| for further output.
     */
    PLDumper	addOct(int i);

    /**
     * Add a real number as a property value. An |Object| reference is
     * used for representation of the real number. The |toString| method of
     * such object is used to get the textual representation.
     * @param	o the object which represents the real number.
     * @return	|PLDumper| for further output.
     */
    PLDumper	addReal(Object o);

    /**
     * Add a Xerox face code as a property value.
     * @param	f the numeric Xerox face code.
     * @return	|PLDumper| for further output.
     */
    PLDumper	addFace(int f);

    /**
     * Add a character string as a property value.
     * @param	s the string to be added to current list.
     * @return	|PLDumper| for further output.
     */
    PLDumper	addStr(String s);

    /**
     * Add a character code as a property value.
     * @param	s the character code to be added to current list.
     * @return	|PLDumper| for further output.
     */
    PLDumper	addChar(short c);

    /**
     * Add a boolean as a property value.
     * @param	b the boolean value to be added to current list.
     * @return	|PLDumper| for further output.
     */
    PLDumper	addBool(boolean b);

    /**
     * Tells the |PLDumper| that it should use numeric notation for
     * character codes even in cases where it would output the symbolic form.
     */
    void	forceNumChars();

}
