/* $Id: AccentPrim.java,v 1.2 2001/09/09 22:17:59 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.typo;

import	nts.base.Dimen;
import	nts.io.CharCode;
import	nts.node.Node;
import	nts.node.AccKernNode;
import	nts.node.HBoxNode;
import	nts.node.HShiftNode;
import	nts.node.FontMetric;
import	nts.builder.Builder;
import	nts.command.Token;
import	nts.command.Command;
import	nts.command.Prim;

public class	AccentPrim	extends BuilderPrim {

    public	AccentPrim(String name) { super(name); }

    /* TeXtp[1122-1125] */
    public final Action		NORMAL = new Action() {
	public void	exec(final Builder bld, Token src) {
	    CharCode	code = Token.makeCharCode(Prim.scanCharacterCode());
	    if (code == CharCode.NULL)
		throw new RuntimeException("no char number scanned");
	    FontMetric	metric = getCurrFontMetric();
	    Node	node = metric.getCharNode(code);
	    if (node == Node.NULL) charWarning(metric, code);
	    else {
		/* Bernd Raichle:
		   The function nextNonAssignment() can execute assignments
		   as in \accent <accent_char> [<assignments>] <base_char>
		   and these assignments can change parameters like
		   the current x height (\fontdimen5\font) and slant
		   (\fontdimen1\font).  Thus we have to get the x height and
		   slant _before_ calling nextNonAssignment()!
		*/
		Dimen	x = metric.getDimenParam(
		                    FontMetric.DIMEN_PARAM_X_HEIGHT);
		Dimen   s = metric.getDimenParam(
		                    FontMetric.DIMEN_PARAM_SLANT);
		Token	tok = nextNonAssignment();
		Command	cmd = meaningOf(tok);
		code = cmd.charCodeToAdd();
		if (code == CharCode.NULL) backToken(tok);
		else {
		    FontMetric	nextMetric = getCurrFontMetric();
		    Node	nextNode = nextMetric.getCharNode(code);
		    if (nextNode == Node.NULL) charWarning(nextMetric, code);
		    else { 
		        Dimen	h = nextNode.getHeight();
			if (!h.equals(x)) {
			    node = HBoxNode.packedOf(node);
			    node = HShiftNode.shiftingDown(node, x.minus(h));
			}
		        Dimen	a = node.getWidth();
			Dimen	delta = makeDelta(a, x, s,
				    nextNode.getWidth(), h,
				    nextMetric.getDimenParam(
					FontMetric.DIMEN_PARAM_SLANT));
			bld.addNode(new AccKernNode(delta));
			bld.addNode(node); node = nextNode;
			bld.addNode(new AccKernNode(delta.plus(a).negative()));
		    }
		}
		bld.addNode(node);
		bld.resetSpaceFactor();
	    }
	}
    };

    private static Dimen	makeDelta(Dimen a, Dimen x, Dimen s,
    					  Dimen w, Dimen h, Dimen t) {
	return Dimen.valueOf(w.minus(a).toDouble() / 2
			     + h.toDouble() * t.toDouble()
			     - x.toDouble() * s.toDouble());
    }

/*
    private static Dimen	makeDelta(Dimen a, Dimen x, Dimen s,
    					  Dimen w, Dimen h, Dimen t)
	{ return w.minus(a).over(2).plus(h.times(t)).minus(x.times(s)); }
*/

    private static int		spoints(Dimen d)
	{ return d.toInt(Dimen.REPR_UNITY); }

}
