/* $Id: BuilderCommand.java,v 1.1.1.1 2001/03/22 15:55:21 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.typo;

import	java.io.ObjectInputStream;
import	java.io.ObjectOutputStream;
import	java.io.Serializable;
import	java.io.IOException;
import	nts.base.Glue;
import	nts.base.ClassAssoc;
import	nts.io.CharCode;
import	nts.builder.Builder;
import	nts.command.Token;

public abstract class	BuilderCommand	extends TypoCommand {

    private static ClassAssoc	assoc;
    private static Object	charHandlerMark;

    public static void		makeStaticData() {
	assoc = new ClassAssoc(Builder.class);
	charHandlerMark = new Serializable() {
	    public int	hashCode() { return 53479; }
	};
    }

    public static void		writeStaticData(ObjectOutputStream output)
    					throws IOException {
	output.writeObject(assoc);
	output.writeObject(charHandlerMark);
    }

    public static void		readStaticData(ObjectInputStream input)
				throws IOException, ClassNotFoundException {
	assoc = (ClassAssoc) input.readObject();
	charHandlerMark = input.readObject();
    }

    public static void		registerBuilder(Class bld)
	{ assoc.record(bld); }

    public final BuilderCommand	defineAction(Class bld, Action act)
	{ assoc.put(bld, this, act); return this; }

    public final Action		getAction(Class bld)
	{ return (Action) assoc.get(bld, this); }

    public final void		exec(Token src) {
        Builder		bld = getBld();
        Action		act = getAction(bld.getClass());
	if (act == Action.NULL) exec(bld, src);
	else act.exec(bld, src);
    }

    public final Glue		getSkipForLeaders() {
        Builder		bld = getBld();
        Action		act = getAction(bld.getClass());
	return (act == Action.NULL)
	     ? getSkipForLeaders(bld) : act.getSkipForLeaders(bld);
    }

    public void		exec(Builder bld, Token src) { illegalCase(bld); }

    public Glue		getSkipForLeaders(Builder bld) { return Glue.NULL; }

    /* ****************************************************************** */

    public static void		defineCharHandler(Class bld, CharHandler hnd)
	{ assoc.put(bld, charHandlerMark, hnd); }

    public static CharHandler	getCharHandler(Class bld)
	{ return (CharHandler) assoc.get(bld, charHandlerMark); }

    public static void		handleChar(CharCode code, Token src) {
        Builder		bld = getBld();
        CharHandler	hnd = getCharHandler(bld.getClass());
	if (hnd == CharHandler.NULL)
	    error("CantUseIn", str("character"), bld);
	else hnd.handle(bld, code, src);
    }

    public static void		handleSpace(Token src) {
        Builder		bld = getBld();
        CharHandler	hnd = getCharHandler(bld.getClass());
	if (hnd == CharHandler.NULL)
	    error("CantUseIn", str("space"), bld);
	else hnd.handleSpace(bld, src);
    }

}
