/* $Id: DiscretionaryPrim.java,v 1.1.1.1 2000/06/07 04:58:39 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.typo;

import	nts.base.Num;
import	nts.io.CharCode;
import	nts.node.Node;
import	nts.node.NodeList;
import	nts.node.DiscretionaryNode;
import	nts.node.FontMetric;
import	nts.builder.Builder;
import	nts.builder.HBoxBuilder;
import	nts.command.Token;
import	nts.command.SimpleGroup;

public class	DiscretionaryPrim	extends BuilderPrim {

    public	DiscretionaryPrim(String name) { super(name); }

    /* TeXtp[1117,1119] */
    public final Action		NORMAL = new Action() {
	public void	exec(final Builder bld, Token src) {
	    bld.addNode(DiscretionaryNode.EMPTY);
	    pushLevel(new DiscGroup());
	}
    };

    public class	DiscGroup	extends SimpleGroup {

	private NodeList[]		lists = new NodeList[3];
	private HBoxBuilder		builder;
	private int			index = 0;

	{ lists[0] = lists[1] = lists[2] = NodeList.EMPTY; }

	public void		start() {
	    builder = new HBoxBuilder(currLineNumber());
	    Builder.push(builder); scanLeftBrace();
	}

	/* TeXtp[1119-1121] */
	public void		close() {
	    Builder.pop();
	    NodeList		list = builder.getList();
	    for (int i = 0; i < list.length(); i++)
		if (!list.nodeAt(i).canBePartOfDiscretionary()) {
		    error("ImproperDisc");
		    addItemsOnDiagLog(
		      "The following discretionary sublist has been deleted:",
				      list.nodes(i));
		    list = new NodeList(list.nodes(0, i));
		    break;
		}
	    lists[index] = list;
	    Builder		bld = getBld();
	    if (++index < lists.length) pushLevel(this);
	    else {
		if (bld.forbidsThirdPartOfDiscretionary() && !list.isEmpty()) {
		    error("IllegalMathDisc", DiscretionaryPrim.this);
		    lists[index - 1] = NodeList.EMPTY;
		} else if (list.length() > DiscretionaryNode.MAX_LIST_LENGTH) {
		    error("TooLongDisc");
		    bld.removeLastNode();
		    bld.addNode(new DiscretionaryNode(lists[0], lists[1],
						      NodeList.EMPTY));
		    bld.addNodes(list.nodes());
		    return;
		}
	    }
	    bld.removeLastNode();
	    bld.addNode(new DiscretionaryNode(lists[0], lists[1], lists[2]));
	}

    }

}
