/* $Id: FontPrim.java,v 1.1.1.1 2001/02/26 00:01:31 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.typo;

import	nts.base.Num;
import	nts.base.Dimen;
import	nts.io.CharCode;
import	nts.io.Name;
import	nts.node.FontMetric;
import	nts.command.FileName;
import	nts.command.Token;
import	nts.command.CtrlSeqToken;
import	nts.command.TokenList;
import	nts.command.PrefixPrim;

public class	FontPrim	extends TypoAssignPrim
				implements TokenList.Provider {

    public FontPrim(String name) { super(name); }

    /* STRANGE
     * ensureOpenLog() is in TeX to prevent the font file
     * name from becoming a job name. Is it possible?
     * First line of the form \font\tenrm=cmr10 \input document.tex
     * assigns texput to logname now, but would it assign cmr10 instead of
     * document otherwise?
     */
    /* TeXtp[1257] */
    protected void		assign(Token src, boolean glob) {
        ensureOpenLog();
        Token		tok = PrefixPrim.definableToken();
	Name		ident = makeIdent(tok);
	tok.define(NullFontMetric.COMMAND, glob); skipOptEquals();
	boolean		inpEnbl = getConfig().enableInput(false);
	FileName	name = scanFileName();
	Dimen		size = scanSizeSpec();
	Num		scale = (size != Dimen.NULL)
			      ? Num.NULL : scanScaleSpec();
	getConfig().enableInput(inpEnbl);
	FontMetric	metric = getTypoHandler()
				    .getMetric(name, size, scale, ident, tok);
	if (metric != FontMetric.NULL)
	    tok.define(new SetFont(metric), glob);
    }

    private static final Name		NAME_FONT = Token.makeName("FONT");

    private Name	makeIdent(Token tok) {
        Name		ident = tok.controlName();
	if (ident == Name.NULL) {
	    CharCode	code = tok.charCode();
	    if (code != CharCode.NULL) {
		Name.Buffer	buf = new Name.Buffer(NAME_FONT.length() + 1);
		buf.append(NAME_FONT).append(code);
		ident = buf.toName();
	    }
	} else if (ident.length() == 0) ident = NAME_FONT;
	return ident;
    }

    private Dimen	scanSizeSpec() {
        Dimen		size = Dimen.NULL;
        if (scanKeyword("at")) {
	    size = scanDimen();
	    if (!size.moreThan(0) || !size.lessThan(2048)) {
		error("ImproperAt", str(size));
		size = Dimen.valueOf(10);	//XXX literals, err strings
	    }
	}
	return size;
    }

    private Num		scanScaleSpec() {
        Num		scale = Num.NULL;
        if (scanKeyword("scaled")) {
	    scale = scanNum();
	    if (!scale.moreThan(0) || scale.moreThan(32768)) {
		error("IllegalMag", str(scale));
		scale = Num.valueOf(1000);	//XXX literals, err strings
	    }
	}
	return scale;
    }

    public boolean	hasFontTokenValue() { return true; }
    public boolean	hasFontMetricValue() { return true; }

    /* TeXtp[415,465] */
    public Token	getFontTokenValue()
	{ return new CtrlSeqToken(getCurrFontMetric().getIdent()); }

    /* TeXtp[577] */
    public FontMetric	getFontMetricValue() { return getCurrFontMetric(); }

}
