/* $Id: NullFontMetric.java,v 1.1.1.1 2001/01/28 17:57:21 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.typo;

import	nts.base.Num;
import	nts.base.Dimen;
import	nts.base.Glue;
import	nts.io.Log;
import	nts.io.CharCode;
import	nts.io.Name;
import	nts.node.Box;
import	nts.node.Node;
import	nts.node.TreatNode;
import	nts.node.WordBuilder;
import	nts.node.WordRebuilder;
import	nts.node.MathWordBuilder;
import	nts.node.BaseFontMetric;
import	nts.command.Token;

public class	NullFontMetric	extends BaseFontMetric {

    public static final NullFontMetric	METRIC = new NullFontMetric();
    public static final SetFont		COMMAND = new SetFont(METRIC);
    public static final Name		NAME = Token.makeName("nullfont");

    public NullFontMetric() {
        setNumParam(NUM_PARAM_HYPHEN_CHAR, Num.valueOf('-'));
        setNumParam(NUM_PARAM_SKEW_CHAR, Num.valueOf(-1));
    }

    public Name		getIdent() { return NAME; }
    public Name		getName() { return NAME; }
    public void		addDescOn(Log log) { NAME.addOn(log); }
    public boolean	isNull() { return true; }
    public Node		getCharNode(CharCode code) { return Node.NULL; }
    public Node		getLargerNode(CharCode code) { return Node.NULL; }

    public Node		getSufficientNode(CharCode code, Dimen desired)
	{ return Node.NULL; }

    public Box		getFittingWidthBox(CharCode code, Dimen desired)
	{ return Box.NULL; }

    public Dimen	getKernBetween(CharCode left, CharCode right)
	{ return Dimen.NULL; }

    					//CCC jikes rejects boundary
    public WordBuilder	getWordBuilder(TreatNode proc, boolean bound,
				       boolean discretionaries)
	{ return new NullWordBuilder(); }

    public WordRebuilder	getWordRebuilder(TreatNode proc,
					         boolean bound)
	{ return new NullWordBuilder(); }

    public MathWordBuilder	getMathWordBuilder(TreatNode proc)
	{ return new NullWordBuilder(); }

    private class	NullWordBuilder
		    implements WordRebuilder, MathWordBuilder {

	public boolean	add(CharCode code) { return false; }
	public byte	addIfBelongsToCut(CharCode code) { return INDEPENDENT; }
	public boolean	prolongsCut(CharCode code) { return false; }
	public void	close(CharCode code) { }
	public void	close(boolean boundary) { }
	public void	close() { }
	public boolean	lastHasCollapsed() { return false; }
	public Node	takeLastNode() { return Node.NULL; }
	public Node	takeLastLargerNode() { return Node.NULL; }

    }

}
