/* $Id: ParShapeParam.java,v 1.1.1.1 2000/04/19 00:20:04 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.typo;

import	nts.base.Num;
import	nts.base.Dimen;
import	nts.io.Log;
import	nts.command.Token;
import	nts.command.ParamPrim;

/**
 * Setting paragraph shape parameter primitive.
 */
public class	ParShapeParam	extends ParamPrim
				implements ParShape.Provider, Num.Provider {

    private ParShape		value;

    /**
     * Creates a new ParShapeParam with given name and value and stores it
     * in language interpreter |EqTable|.
     * @param	name the name of the ParShapeParam
     * @param	val the value of the ParShapeParam
     */
    public	ParShapeParam(String name, ParShape val)
	{ super(name); value = val; }

    /**
     * Creates a new ParShapeParam with given name and stores it
     * in language interpreter |EqTable|.
     * @param	name the name of the ParShapeParam
     */
    public	ParShapeParam(String name) { this(name, ParShape.EMPTY); }

    public final Object		getEqValue() { return value; }

    public final void	setEqValue(Object val) { value = (ParShape) val; }

    public final void	addEqValueOn(Log log)
	{ log.add(value.getLength()); }

    public final ParShape	get() { return value; }

    public void		set(ParShape val, boolean glob)
	{ beforeSetting(glob); value = val; }

    protected void	scanValue(Token src, boolean glob)
	{ set(scanParShapeValue(src), glob); }

    /* TeXtp[1248] */
    protected ParShape	scanParShapeValue(Token src) {
	int		len = scanInt();
	if (len <= 0) return ParShape.EMPTY;
	Dimen[]		indents = new Dimen[len];
	Dimen[]		widths = new Dimen[len];
	for (int i = 0; i < len; i++)
	    { indents[i] = scanDimen(); widths[i] = scanDimen(); }
	return new ParShape(indents, widths);
    }

    public ParShape	getParShapeValue() { return get(); }
    public boolean	hasNumValue() { return true; }
    public Num		getNumValue()
	{ return Num.valueOf(get().getLength()); }

}
