/* $Id: SetBoxPrim.java,v 1.1.1.1 2001/04/18 06:16:27 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.typo;

import	nts.io.Log;
import	nts.node.Box;
import	nts.node.VoidBoxNode;
import	nts.node.NodeEnum;
import	nts.node.TreatBox;
import	nts.command.Token;
import	nts.command.RegisterPrim;

public class	SetBoxPrim	extends RegisterPrim {

    private final String	desc;

    /**
     * Creates a new |SetBoxPrim| with given name and stores it
     * in language interpreter |EqTable|.
     * @param	name the name of the |SetBoxPrim|
     */
    public SetBoxPrim(String name, String desc)
	{ super(name); this.desc = desc; }

    public String	getDesc() { return desc; }
    public String	getEqDesc() { return desc; }

    public final void	set(int idx, Box val, boolean glob) {
	if (glob) getEqt().gput(tabKind, idx, val);
	else getEqt().put(tabKind, idx, val);
    }

    public final Box	get(int idx) {
	Box	val =  (Box) getEqt().get(tabKind, idx);
	return (val != Box.NULL) ? val : VoidBoxNode.BOX;
    }

    public final Box	steal(int idx) {
	Box	val =  (Box) getEqt().get(tabKind, idx);
	if (val != Box.NULL) {
	    getEqt().nastyReplace(tabKind, idx, VoidBoxNode.BOX);
	    return val;
	}
	return VoidBoxNode.BOX;
    }

    public final void	foist(int idx, Box val) {
	/* if (getEqt().get(tabKind, idx) != Box.NULL) */
	getEqt().nastyReplace(tabKind, idx, val);
    }

    public void		addEqValueOn(int idx, Log log)
	{ get(idx).addOn(log, 0, 1); }

    /**
     * Performs the assignment.
     * @param	src source token for diagnostic output.
     * @param	glob indication that the assignment is global.
     */
    protected final void	assign(Token src, final boolean glob) {
	final int	idx = scanRegisterCode();
        skipOptEquals();
	TypoCommand.scanBox(new TreatBox() {
	    public void	execute(Box box, NodeEnum mig)
		{ set(idx, box, glob); }
	});
    }

    /* STRANGE
     * Why it scans code and equals when \setbox is not allowed
     */
    public void			doAssignment(Token src, int prefixes) {
	beforeAssignment(this, prefixes);
	scanRegisterCode(); skipOptEquals();
	error("ImproperSetbox", this);
	afterAssignment();
    }

    public void			showAndPurge(int idx) {
	//XXX maybe addBoxOnDiagLog(String, Box) could by used but only
	//XXX if it is certain that error() ends the previous line in diagLog
        diagLog.add("The following box has been deleted:");
	TypoCommand.addBoxOnDiagLog(get(idx));
	foist(idx, VoidBoxNode.BOX);
    }

}
