/* $Id: SetFont.java,v 1.1.1.1 2001/03/22 13:35:51 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.typo;

import	nts.io.Log;
import	nts.node.FontMetric;
import	nts.command.Token;
import	nts.command.CtrlSeqToken;
import	nts.command.TokenList;
import	nts.command.Command;
import	nts.command.PrefixPrim;

//XXX maybe create PrefixedCommand and derive PrefixPrim of it
//XXX maybe create AssignCommand and derive AssignPrim of it

public class	SetFont		extends TypoCommand
				implements TokenList.Provider {

    private final FontMetric		metric;

    public SetFont(FontMetric metric) { this.metric = metric; }

    /** Non prefixed version of exec */
    public final void		exec(Token src) { exec(src, 0); }

    public final boolean	assignable() { return true; }
    public final void		doAssignment(Token src, int prefixes)
				    { exec(src, prefixes); }

    /**
     * Performs itself in the process of interpretation of the macro language
     * after sequence of prefix commands.
     * @param	src source token for diagnostic output.
     * @param	prefixes accumulated code of prefixes.
     */
    /* TeXtp[1217] */
    public final void		exec(Token src, int prefixes) {
	PrefixPrim.beforeAssignment(this, prefixes);
	setCurrFontMetric(metric, PrefixPrim.globalAssignment(prefixes));
	PrefixPrim.afterAssignment();
    }

    /* TeXtp[1261] */
    public final void		addOn(Log log)
	{ log.add("select font "); metric.addDescOn(log); }

    public boolean		sameAs(Command cmd) {
	return (  cmd instanceof SetFont
	       && metric.equals(((SetFont) cmd).metric)  );
    }

    public boolean	hasFontTokenValue() { return true; }
    public boolean	hasFontMetricValue() { return true; }

    /* STRANGE
     * What is \FONT~ good for?
     */
    /* TeXtp[415,465] */
    public Token	getFontTokenValue()
	{ return new CtrlSeqToken(metric.getIdent()); }

    /* TeXtp[577] */
    public FontMetric	getFontMetricValue() { return metric; }

}
