/* $Id: SpaceFactorPrim.java,v 1.1.1.1 1999/08/03 13:54:10 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.typo;

import	nts.base.Num;
import	nts.base.Dimen;
import	nts.builder.Builder;
import	nts.command.Token;

public class	SpaceFactorPrim	extends TypoAssignPrim
				implements Num.Provider {

    public SpaceFactorPrim(String name) { super(name); }

    public static final int	MIN_SPACE_FACTOR = 1;
    public static final int	MAX_SPACE_FACTOR = 32767;

    /* STRANGE
     * \global\spacefactor is allowed but has no effect
     */
    /* TeXtp[1243] */
    protected void	assign(Token src, boolean glob) {
        Builder		bld = getBld();
	if (bld.getSpaceFactor() > 0) {
	    skipOptEquals();
	    int		sf = scanInt();
	    if (MIN_SPACE_FACTOR <= sf && sf <= MAX_SPACE_FACTOR)
		bld.setSpaceFactor(sf);
	    else error("BadSpaceFactor", num(sf),
		       num(MIN_SPACE_FACTOR), num(MAX_SPACE_FACTOR));
	}
	else illegalCase(bld);
    }

    public boolean	hasCrazyValue() {
	if (getBld().getSpaceFactor() > 0) return false;
	else { error("ImproperSForPD", this); return true; }
    }

    public boolean	hasDimenValue()
	{ return !(getBld().getSpaceFactor() > 0); }

    public Dimen	getDimenValue() {
	int		sf = getBld().getSpaceFactor();
	if (sf > 0) return Dimen.NULL;
	else { error("ImproperSForPD", this); return Dimen.ZERO; }
    }

    public boolean	hasNumValue() { return true; }

    /* TeXtp[418] */
    public Num	getNumValue() {
	int		sf = getBld().getSpaceFactor();
	if (sf > 0) return Num.valueOf(sf);
	else { error("ImproperSForPD", this); return Num.ZERO; }
    }

}
