/* $Id: WritePrim.java,v 1.1.1.1 2000/05/27 02:14:18 ksk Exp $
 *
 * NTS: The New Typesetting System (modular reimplementation of TeX)
 * Copyright (C) 2001  Karel Skoupy
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * email: ksk@dante.de
 * snail: Karel Skoupy
 *        c/o DANTE, Deutschsprachige Anwendervereinigung TeX e.V.
 *        Postfach 10 18 40, 69008 Heidelberg, Germany
 * www:   http://www.dante.de
 */
package	nts.typo;

import	java.io.ObjectInputStream;
import	java.io.IOException;
import	java.util.HashMap;
import	nts.io.Log;
import	nts.io.CntxLog;
import	nts.node.BaseNode;
import	nts.node.TypeSetter;
import	nts.node.SettingContext;
import	nts.builder.Builder;
import	nts.builder.NullBuilder;
import	nts.command.Token;
import	nts.command.TokenList;
import	nts.command.FrozenToken;
import	nts.command.LeftBraceToken;
import	nts.command.RightBraceToken;
import	nts.command.Macro;
import	nts.command.MacroBody;
import	nts.command.Prim;
import	nts.command.PrefixPrim;

public class	WritePrim extends BuilderPrim {

    private transient HashMap		table;

    private void	initTable() { table = new HashMap(23); }

    public WritePrim(String name) { super(name); initTable(); }

    private void	readObject(ObjectInputStream input)
				throws IOException, ClassNotFoundException
	{ input.defaultReadObject(); initTable(); }

    public Log		get(int num)
	{ return (Log) table.get(new Integer(num)); }

    private Log		replace(int num, Log output) {
	return (Log) table.put(new Integer(num), output);
	//XXX remove if NULL
    }

    public void		set(int num, Log output) {
	Log		old = replace(num, output);
	if (old != output && old != Log.NULL) old.close();
    }

    public void		exec(Builder bld, Token src) {
	final int		num = scanInt();
	final TokenList		list = Prim.scanTokenList(src, false);
	bld.addNode(new WriteNode(num, list));
    }

    /* TeXtp[1352] */
    public boolean	immedExec(Token src) {
	int		num = scanInt();
	TokenList	list = Prim.scanTokenList(src, false);
	write(num, list); return true;
    }

    private static final Token		FROZEN_END_WRITE
	= new FrozenToken("endwrite",
			  new Macro(MacroBody.EMPTY, PrefixPrim.OUTER));

    private static final Token[]	END_TOKS
	= { RightBraceToken.TOKEN, FROZEN_END_WRITE };

    private static final TokenList	END_LIST = new TokenList(END_TOKS);

    private static final NullBuilder	NULL_BUILDER = new NullBuilder();

    /* TeXtp[1370] */
    protected void	write(int num, TokenList list) {
	insertList(END_LIST);
	tracedPushXList(list, "write");
	insertTokenWithoutCleaning(LeftBraceToken.TOKEN);
	Builder.push(NULL_BUILDER);
	// list = Prim.scanTokenList(esc(getName()), true);
	list = Prim.scanTokenList(this, true);
	Token	tok = nextRawToken();
	if (tok != FROZEN_END_WRITE) {
	    error("UnbalancedWrite");
	    do tok = nextRawToken();
	    while (tok != FROZEN_END_WRITE);
	}
	Builder.pop();
	getTokStack().dropFinishedPop();
	Log		output;
	if (num < 0) output = fileLog.startLine();
	else if ((output = get(num)) == Log.NULL)
	    output = normLog.startLine();
	output.add(list).endLine();
    }

    protected abstract static class	FileNode	extends BaseNode {
	/* root corresponding to whatsit_node */

	protected int			num;

	public FileNode(int num) { this.num = num; }

	public boolean	sizeIgnored() { return true; }

	/* TeXtp[1355] */
	protected Log	addName(Log log, String name) {
	    log.addEsc(name);
	    return (num < 0) ? log.add('-')
		: (num > getConfig().getIntParam(Prim.INTP_MAX_FILE_CODE))
		? log.add('*') : log.add(num);
	}

	public byte	beforeWord() { return SKIP; }
	public byte	afterWord() { return SUCCESS; }

    }

    protected class	WriteNode	extends FileNode {
	/* corresponding to whatsit_node */

	protected TokenList		list;

	public WriteNode(int num, TokenList list)
	    { super(num); this.list = list; }

	/* TeXtp[1356] */
	public void	addOn(Log log, CntxLog cntx)
	    { addNodeToks(addName(log, "write"), list); }

	/* TeXtp[1366, 1367] */
	public void	typeSet(TypeSetter setter, SettingContext sctx)
	    { if (sctx.allowIO) write(num, list); }

    }

}
